<?php
require '../../vendor/autoload.php';
require 'wintux.php';
require 'rmh.php';
require 'log.php';
include 'config.php';

use Lcobucci\JWT\Builder;
use Lcobucci\JWT\Signer\Hmac\Sha256;
use Lcobucci\JWT\ValidationData;
use Lcobucci\JWT\Parser;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;

// $job1 = new \Cron\Job\ShellJob();
// $job1->setCommand('ls -la ../v1'); //php public_html/dev/api/lfa/v1/codegen.php 20efabe5-8c8a-11e9-8f13-deadbe115600
// $job1->setSchedule(new \Cron\Schedule\CrontabSchedule('*/1 * * * *')); // 30 8 * * 1-6 * runs every day monday to saturday at 8:30 am

// $resolver = new \Cron\Resolver\ArrayResolver();
// $resolver->addJob($job1);

// $cron = new \Cron\Cron();
// $cron->setExecutor(new \Cron\Executor\Executor());
// $cron->setResolver($resolver);

$app = new Slim\App(["settings" => $config]);


//Handle Dependencies
$container = $app->getContainer();

$container['apidb'] = function ($c) {
   
   try {
       $db = $c['settings']['apidb'];
       $options  = array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION, PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC, );
       $pdo = new PDO("mysql:host=" . $db['servername'] . ";dbname=" . $db['dbname'], $db['username'], $db['password'],$options);
       return $pdo;
   }
   catch(\Exception $ex) {
       return $ex->getMessage();
   }
};

$container['webstoredb_dev'] = function ($c) {
   
    try{
        $db = $c['settings']['webstoredb_dev'];
        $options  = array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION, PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC, );
        $pdo = new PDO("mysql:host=" . $db['servername'] . ";dbname=" . $db['dbname'], $db['username'], $db['password'],$options);
        return $pdo;
    }
    catch(\Exception $ex) {
        return $ex->getMessage();
    }
 };

 $container['webstoredb'] = function ($c) {
   
    try{
        $db = $c['settings']['webstoredb'];
        $options  = array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION, PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC, );
        $pdo = new PDO("mysql:host=" . $db['servername'] . ";dbname=" . $db['dbname'], $db['username'], $db['password'],$options);
        return $pdo;
    }
    catch(\Exception $ex) {
        return $ex->getMessage();
    }
 };

 $container['m2mkioskdb'] = function ($c) {
   
    try{
        $db = $c['settings']['m2mkioskdb'];
        $options  = array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION, PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC, );
        $pdo = new PDO("mysql:host=" . $db['servername'] . ";dbname=" . $db['dbname'], $db['username'], $db['password'],$options);
        return $pdo;
    }
    catch(\Exception $ex) {
        return $ex->getMessage();
    }
 };

 $mw_authorize = function( $request, $response, $next ) {
    // if the client is the POS continue and don't validate user
    // return $response->withJson( array( 'status' => false, 'message' => $request->getAttribute('client') ), 401 );
    if( $request->getAttribute('client')['client_type_id'] == 4 ) {
        $response = $next($request, $response);
        return $response;
    }


    $auth_header = $request->getHeaderLine('authorization');

    $auths = explode(',', $auth_header );
    $auth_token;

    if(count($auths) < 2)
        return $response->withJson( array( 'status' => false, 'message' => 'Access Denied.', 'signout' => true ), 401 );

    $auth_token = str_replace('Bearer ', '', $auths[1]);
    $token = '';

    // Validate the user token
    if ( isset( $auth_token ) ) {
        try {
            $token = (new Parser())->parse((string) $auth_token); // Parses from a string
            $token->getHeaders(); // Retrieves the token header
            $token->getClaims(); // Retrieves the token claims
            
            $usr = $token->getClaim('usr');
            // $prv = $token->getClaim('prv');

            $data = new ValidationData(); // It will use the current time to validate (iat, nbf and exp)
            $data->setIssuer('https://leonardofifthavenue.com');
            $data->setAudience('lfa-rental-kiosk');
            $data->setId('4f1g23a12aa');
            
            if ( !$token->validate($data) )
                return $response->withJson( array( 'status' => false, 'message' => 'Access Denied.', 'signout' => true ), 401 );

            // generate new token with new expiration date
            $signer = new Sha256();
            $secret_key = 'LF@!-' . $usr->employeeId . '-' . time();
            $token = (new Builder())->setIssuer('https://leonardofifthavenue.com') // Configures the issuer (iss claim)
                            ->setAudience('lfa-rental-kiosk') // Configures the audience (aud claim)
                            ->setId('4f1g23a12aa', true) // Configures the id (jti claim), replicating as a header item
                            ->setIssuedAt(time()) // Configures the time that the token was issue (iat claim)
                            ->setNotBefore(time()) // Configures the time that the token can be used (nbf claim)
                            ->setExpiration(time() + 900 ) // Configures the expiration time of the token (exp claim) (15 mins)
                            ->set('usr', $usr) // Configures a new claim, called "username"
                            ->sign($signer, $secret_key) // creates a signature using "testing" as key
                            ->getToken(); // Retrieves the generated token
        }
        catch(\Exception $ex) {
            return $response->withJson( array( 'error' => true, 'message' => 'Could not authenticate.' ), 400 );
        }
    }

    $request = $request->withAttribute('token', $token->__toString());
    $response = $next($request, $response);
    return $response;
};

// Add the authentication function to the API execution stack 
$app->add( function ( $request, $response, $next ) {
    
    if( $request->isOptions() ) {
        $request = $request->withAttribute('client', $result);
        $response = $next($request, $response);
        return $response;
    }

    $auth_header = $request->getHeaderLine('authorization');
    // return $response->withJson( array( 'error' => true, 'message' => $auth_header ), 200 );
    $auths = explode(',', $auth_header );
    $auth_key;

    if(count($auths) < 2)
        $auth_key = $auths[0];
    else {
        $auth_key = $auths[0];
    }

    // Validate the Authorization Key
    if ( isset( $auth_key ) ) {
		
        try{
            $con = $this->apidb;
            $sql = "SELECT id, type_id client_type_id, client_type type, '' wintuxStoreId FROM vwClient c
                    where type_id = 1
                        and active = 1
                        and code = :auth_key
                    union all
                    select id, typeId client_type_od, type, wintuxStoreId from vwStore s 
                    where typeId = 5
                        and active = 1
                        and authKey = :auth_key;";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array( ':auth_key' => $auth_key );
            $pre->execute($values);
            $result = $pre->fetch();
        }
        catch(\Exception $ex) {
            return $response->withJson( array( 'error' => true, 'message' => 'Could not authenticate.' ), 400 );
        }
		
        if ( !$result ) {
            return $response->withJson( array( 'error' => true, 'message' => 'Invalid Api key.' ), 401 );
            // $app->stop();
        } 

        $request = $request->withAttribute('client', $result);
        $response = $next($request, $response);
        return $response;
		
    } else {
        return $response->withJson( array( 'error' => true, 'message' => 'Api key is misssing.' ), 400 );
        // $app->stop();
    }
});

$app->options('/{routes:.+}', function ($request, $response, $args) {
    return $response;
});

$app->add(function ($req, $res, $next) {
    $response = $next($req, $res);

    return $response
            ->withHeader('Access-Control-Allow-Origin', '*')
            ->withHeader('Access-Control-Allow-Headers', 'X-Requested-With, Content-Type, Accept, Origin, Authorization')
            ->withHeader('Access-Control-Allow-Methods', 'GET, POST, PUT, DELETE, PATCH, OPTIONS');
});

// Middleware to get clients IP
$checkProxyHeaders = false; 
$trustedProxies = [];
$app->add(new RKA\Middleware\IpAddress($checkProxyHeaders, $trustedProxies));

// Let the world know that we are live!
$app->get('/', function ($request, $response) use ($lrPluginUrl) {
    return $response->withJson(array('status' => 'active','message'=> 'Welcome to Leonardo Fifth Avenue API.'), 200);
});

/*************************************
*************** SEARCH ***************
*************************************/

/*
 * Get list of those customer, events and reservations that matches the identifier query
 * 
 * identifier:  varchar(15)                             // query string
 */
$app->get('/search/{identifier}', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;

        // customers
        $sql = "select c.customerId, customerTypeId, customerType, firstName, lastName, fullName, phone, email, gender, dateCreated, addressLine1, addressLine2, addressCity, addressState, addressZipcode, ck.memberYears, ck.d2mOrders, ck.rentShopOrders, 'Customer' category
                from vwCustomer c, vwCustomerKeyItems ck
                where c.customerId = ck.customerId
                      and (phone like :identifier
                      or email like :identifier
                      or fullName like :identifier)
                order by dateCreated desc 
                limit 10";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => '%' . $request->getAttribute('identifier') . '%'
        );
        $pre->execute( $values );
        $customers = $pre->fetchall();

        if( !$customers )
            $customers = array();

        // events
        $sql = "select eventId, occasion, eventType, location, coordinator1FullName, coordinator1Phone, coordinator1Email, eventDate, status, 'Event' category
                from vwEvent 
                where eventId like :identifier
                      or wintuxEventId like :identifier 
                      or coordinator1FullName like :identifier 
                      or coordinator1Phone like :identifier
                      or coordinator2FullName like :identifier 
                      or coordinator2Phone like :identifier
                      or location like :identifier
                      or DATE_FORMAT(eventDate, '%Y-%m-%d') like :identifier
                order by eventDate desc 
                limit 10";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => '%' . $request->getAttribute('identifier') . '%'
        );
        $pre->execute( $values );
        $events = $pre->fetchall();

        if( !$events )
            $events = array();

        // orders
        $sql = "select orderId, foreignId, customerId, customerTypeId, customerType, customerFullName, customerPhone, customerEmail, orderDate, subtotal, otherChargesDiscount, tax, shipping, total, orderStatus, 'Order' category
                from vwOrder 
                where customerFullName like :identifier
                      or customerPhone like :identifier
                      or customerEmail like :identifier
                      or DATE_FORMAT(orderDate, '%Y-%m-%d') like :identifier
                      or orderId like :identifier
                      or foreignId like :identifier
                order by orderDate desc 
                limit 10";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => '%' . $request->getAttribute('identifier') . '%'
        );
        $pre->execute( $values );
        $orders = $pre->fetchall();

        if( !$orders )
            $orders = array();

        return $response->withJson(array('status' => true, 'error' => false, 'data' => array( 'customers' => $customers, 'events' => $events, 'reservations' => array(), 'orders' => $orders ) ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*************************************
************ RESERVATIONS ************
*************************************/

/* 
 * Get list of all the reservations
 */
$app->get('/reservations', function ($request, $response) use ($lrPluginUrl) {
    try {
        $con = $this->apidb;

        // orders
        $sql = "select *
                from vwReservation 
                order by reservationDate desc 
                limit 200";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute();
        $reservations = $pre->fetchall();

        if( !$reservations )
            $reservations = array();

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $reservations ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/* 
 * Get list of those reservations that matches the identifier query
 * 
 * identifier:  varchar(15)                             // query string to look by customer name, customer phone, customer email,
 *                                                         and reservation date
 */
$app->get('/reservations/{identifier}', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;

        // orders
        $sql = "select *, 'Reservation' category
                from vwReservation 
                where customerFullName like :identifier
                      or customerPhone like :identifier
                      or customerEmail like :identifier
                      or DATE_FORMAT(reservationDate, '%Y-%m-%d') like :identifier
                      or reservationId like :identifier
                order by reservationDate desc 
                limit 10";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => '%' . $request->getAttribute('identifier') . '%'
        );
        $pre->execute( $values );
        $reservations = $pre->fetchall();

        if( !$reservations )
            $reservations = array();

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $reservations ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/* 
 * Get list of those reservations that matches the identifier query
 * 
 * identifier:  varchar(15)                             // query string to look by customer name, customer phone, customer email,
 *                                                         and reservation date
 */
$app->get('/reservation/{identifier}', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;

        // find reservation information
        $sql = "select reservationId, reservationDate, orderId, customerId, customerMeasurementId, eventId, useDate, wantDate, returnDate, pickupLocationId, comments
                from vwReservation 
                where reservationId = :identifier";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => $request->getAttribute('identifier')
        );
        $pre->execute( $values );
        $reservation = $pre->fetch();

        if( !$reservation )
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'Reservation not found.' ), 200);

        // get customer information
        $sql = "call spCustomerInformation(:identifier)";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => $reservation['customerId']
        );
        $pre->execute( $values );
        $customer = $pre->fetch();

        if( !$customer ) 
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'Customer doesn\'t exist'), 200);

        $pre->nextRowset();
        $measurements = $pre->fetch();

        $customer['measurements'] = $measurements;

        $sql = "select cm.id measurementId, cm.coat_size coatSize, cm.coat_length coatLength, cm.coat_sleeve coatSleeve,
                        cm.shirt_size shirtSize, cm.pant_waist pantWaist, cm.pant_outseam pantOutseam, cm.shoe_size shoe, 
                        cm.date_created measurementDate
                from leonard5_lfa_main.tblCustomer c left outer join leonard5_lfa_main.tblCustomerRentalMeasurements cm on c.id = cm.customer_id
                where c.id = :identifier and cm.id = :measurements_id;";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier'       => $reservation['customerId'],
            ':measurements_id'  => $reservation['customerMeasurementId']
        );
        $pre->execute( $values );
        $rental_measurements = $pre->fetch();

        $customer['rentalMeasurements'] = $rental_measurements;

        $reservation['customer'] = $customer;

        // find selected products
        $sql = "select product_id
                from tblReservationItems 
                where reservation_id = :reservation_id";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':reservation_id' => $reservation['reservationId']
        );
        $pre->execute( $values );
        $result = $pre->fetchall();

        $reservation_items = array();
        foreach( $result as $item ) {
            array_push( $reservation_items, $item['product_id'] );
        }

        $reservation['items'] = $reservation_items;

        // get event information
        if( $reservation['eventId'] != null ) {
            $sql = "select eventId, occasion, eventType, location, coordinator1FullName, coordinator1Phone, coordinator1Email, eventDate, status
                    from vwEvent 
                    where eventId = :identifier";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':identifier' => $reservation['eventId']
            );
            $pre->execute( $values );
            $event = $pre->fetch();

            $reservation['event'] = $event;
        }
        else {
            $reservation['event'] = null;
        }

        // get promo codes applied
        $sql = "select p.code, p.description, p.use_with_others, p.amount, p.min_amount, p.max_amount, p.min_items, p.max_items
                from tblReservationPromoCodes c, tblPromoCode p
                where c.code = p.code and reservation_id = :identifier";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => $reservation['reservationId']
        );
        $pre->execute( $values );
        $promo_codes = $pre->fetchall();

        $reservation['promoCodes'] = $promo_codes;

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $reservation ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * Create a new reservation
 * 
 * reservation: array(                                                                                              *
 *  customer_id                     int                                   // customer id                            *
 *  customer_measurement_id         int                                   // customer rental measurement id         *
 *  event_id                        int                                   // event id associated with this          
 *                                                                           reservation
 *  event_role_id                   smallint                              // customer role on the event             
 *  use_date                        date                                  // date when the customer will be         *
 *                                                                           using the outfit
 *  want_date                       datetime                              // date and time when the customer        *
 *                                                                           will be picking up the outfit
 *  return_date                     date                                  // date when the customer will be         *
 *                                                                           returning the outfit
 *  pickup_location_id              smallint                              // id of the pickup location              *
 *  comments                        varchar(250)                          // reservation comments                   *
 *  items: array(                                                         // list of products in the reservation    *
 *      product_id                      int                               // id of the product                      *
 *  )
 *  promo_codes: array(                                                   // list of promo codes applied to         *
 *                                                                           reservation
 *      code                            varchar(15)                       // promo code                             *
 *  )
 * )                   
 */
$app->post('/reservation', function ($request, $response) use ($lrPluginUrl) {
    try {

        // Get all the request values
        $params = $request->getParsedBody()['reservation'];
        
        $token = (new Parser())->parse((string) $request->getAttribute('token'));         
        $usr = $token->getClaim('usr');

        // validate reservation
        $errors = array();
        if( empty( $params['customer_id'] ) )
            array_push( $errors, 'You must select a customer' );
        else if( empty( $params['customer_measurement_id'] ) ) 
            array_push( $errors, 'Measurements are required' );

        if( empty( $params['use_date'] ) ) 
            array_push( $errors, 'Use date is required' );

        if( empty( $params['want_date'] ) ) 
            array_push( $errors, 'Want date is required' );
        else if( strtotime( $params['want_date'] ) > strtotime( $params['use_date'] ) )
            array_push( $errors, 'Want date must be before use date' );
        else if( strtotime( $params['want_date'] ) > strtotime( $params['return_date'] ) ) 
            array_push( $errors, 'Want date must be before return date' );

        if( empty( $params['return_date'] ) ) 
            array_push( $errors, 'Return date is required' );
        else if( strtotime( $params['return_date'] ) < strtotime( $params['use_date'] ) ) 
            array_push( $errors, 'Return date must be after use date' );
        else if( strtotime( $params['return_date'] ) < strtotime( $params['want_date'] ) ) 
            array_push( $errors, 'Return date must be after want date' );

        if( empty( $params['pickup_location_id'] ) ) 
            array_push( $errors, 'Pickup location is required' );

        if( count( $params['items'] ) == 0 ) 
            array_push( $errors, 'Items are required' );

        if( count($errors) > 0 ) 
            return $response->withJson(array('status' => false, 'error' => true, 'errors' => $errors ), 200);
        
        // Create reservation
        $con = $this->apidb;
        $sql = "insert into tblReservation (
                                        reservation_date, 
                                        customer_id, 
                                        customer_measurement_id, 
                                        event_id, 
                                        event_role_id, 
                                        use_date, 
                                        want_date, 
                                        return_date, 
                                        pickup_location_id, 
                                        comments, 
                                        employee_id,
                                        store_id
                                        ) values (
                                        now(),
                                        :customer_id,
                                        :customer_measurement_id,
                                        :event_id,
                                        :event_role_id,
                                        :use_date,
                                        :want_date,
                                        :return_date,
                                        :pickup_location_id,
                                        :comments,
                                        :employee_id,
                                        :store_id)"; 
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':customer_id'              => $params['customer_id'],
            ':customer_measurement_id'  => $params['customer_measurement_id'],
            ':event_id'                 => $params['event_id'],
            ':event_role_id'            => $params['event_role_id'],
            ':use_date'                 => $params['use_date'],
            ':want_date'                => $params['want_date'],
            ':return_date'              => $params['return_date'],
            ':pickup_location_id'       => $params['pickup_location_id'],
            ':comments'                 => $params['comments'],
            ':employee_id'              => $usr->id,
            ':store_id'                 => $usr->baseStoreId
        );
        $pre->execute( $values );

        // Get reservation id
        $sql = "SELECT LAST_INSERT_ID() id;"; 
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute();
        $reservation_id = $pre->fetch()['id'];

        // Save reservation items
        foreach( $params['items'] as $product_id ) {
            $sql = "insert into tblReservationItems (
                        reservation_id, 
                        product_id
                        ) values (
                        :reservation_id,
                        :product_id)"; 
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':reservation_id'   => $reservation_id,
                ':product_id'       => $product_id
            );
            $pre->execute( $values );
        }

        // save reservation promo codes
        foreach( $params['promo_codes'] as $code ) {
            $sql = "insert into tblReservationPromoCodes (
                        reservation_id, 
                        code
                        ) values (
                        :reservation_id,
                        :code)"; 
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':reservation_id'   => $reservation_id,
                ':code'             => $code['code']
            );
            $pre->execute( $values );
        }

        return $response->withJson(array('status' => true, 'error' => false, 'reservationId' => $reservation_id ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * Update a reservation
 * 
 * reservation: array(                                                                                              *
 *  customer_id                     int                                   // customer id                            *
 *  customer_measurement_id         int                                   // customer rental measurement id         *
 *  event_id                        int                                   // event id associated with this          
 *                                                                           reservation
 *  event_role_id                   smallint                              // customer role on the event             
 *  use_date                        date                                  // date when the customer will be         *
 *                                                                           using the outfit
 *  want_date                       datetime                              // date and time when the customer        *
 *                                                                           will be picking up the outfit
 *  return_date                     date                                  // date when the customer will be         *
 *                                                                           returning the outfit
 *  pickup_location_id              smallint                              // id of the pickup location              *
 *  comments                        varchar(250)                          // reservation comments                   *
 *  items: array(                                                         // list of products in the reservation    *
 *      product_id                      int                               // id of the product                      *
 *  )
 *  promo_codes: array(                                                   // list of promo codes applied to         *
 *                                                                           reservation
 *      code                            varchar(15)                       // promo code                             *
 *  )
 *  reservation_id                  int                                   // id of the reservation to be updated    *
 * )                   
 */
$app->put('/reservation', function ($request, $response) use ($lrPluginUrl) {
    try {
        // Get all the request values
        $params = $request->getParsedBody()['reservation'];

        // validate reservation
        $errors = array();
        if( empty( $params['reservation_id'] ) )
            array_push( $errors, 'Reservation ID not found' );
        if( empty( $params['customer_id'] ) )
            array_push( $errors, 'You must select a customer' );
        else if( empty( $params['customer_measurement_id'] ) ) 
            array_push( $errors, 'Measurements are required' );

        if( empty( $params['use_date'] ) ) 
            array_push( $errors, 'Use date is required' );

        if( empty( $params['want_date'] ) ) 
            array_push( $errors, 'Want date is required' );
        else if( strtotime( $params['want_date'] ) > strtotime( $params['use_date'] ) )
            array_push( $errors, 'Want date must be before use date' );
        else if( strtotime( $params['want_date'] ) > strtotime( $params['return_date'] ) ) 
            array_push( $errors, 'Want date must be before return date' );

        if( empty( $params['return_date'] ) ) 
            array_push( $errors, 'Return date is required' );
        else if( strtotime( $params['return_date'] ) < strtotime( $params['use_date'] ) ) 
            array_push( $errors, 'Return date must be after use date' );
        else if( strtotime( $params['return_date'] ) < strtotime( $params['want_date'] ) ) 
            array_push( $errors, 'Return date must be after want date' );

        if( empty( $params['pickup_location_id'] ) ) 
            array_push( $errors, 'Pickup location is required' );

        if( count( $params['items'] ) == 0 ) 
            array_push( $errors, 'Items are required' );

        if( count($errors) > 0 ) 
            return $response->withJson(array('status' => false, 'error' => true, 'errors' => $errors ), 200);
        
        // Create reservation
        $con = $this->apidb;
        $sql = "update tblReservation set 
                                        customer_measurement_id = :customer_measurement_id,
                                        event_id = :event_id,
                                        event_role_id = :event_role_id,
                                        use_date = :use_date,
                                        want_date = :want_date,
                                        return_date = :return_date,
                                        pickup_location_id = :pickup_location_id,
                                        comments = :comments
                where id = :reservation_id;"; 
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':reservation_id'           => $params['reservation_id'],
            ':customer_measurement_id'  => $params['customer_measurement_id'],
            ':event_id'                 => $params['event_id'],
            ':event_role_id'            => $params['event_role_id'],
            ':use_date'                 => $params['use_date'],
            ':want_date'                => $params['want_date'],
            ':return_date'              => $params['return_date'],
            ':pickup_location_id'       => $params['pickup_location_id'],
            ':comments'                 => $params['comments']
        );
        $pre->execute( $values );

        // Save reservation items
        $sql = "delete from tblReservationItems where reservation_id = :reservation_id;"; 
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':reservation_id'   => $params['reservation_id']
        );
        $pre->execute( $values );

        foreach( $params['items'] as $product_id ) {
            $sql = "insert into tblReservationItems (
                        reservation_id, 
                        product_id
                        ) values (
                        :reservation_id,
                        :product_id)"; 
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':reservation_id'   => $params['reservation_id'],
                ':product_id'       => $product_id
            );
            $pre->execute( $values );
        }

        // save reservation promo codes
        $sql = "delete from tblReservationPromoCodes where reservation_id = :reservation_id;"; 
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':reservation_id'   => $params['reservation_id']
        );
        $pre->execute( $values );
        foreach( $params['promo_codes'] as $code ) {
            $sql = "insert into tblReservationPromoCodes (
                        reservation_id, 
                        code
                        ) values (
                        :reservation_id,
                        :code)"; 
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':reservation_id'   => $params['reservation_id'],
                ':code'             => $code['code']
            );
            $pre->execute( $values );
        }

        return $response->withJson(array('status' => true, 'error' => false, 'message' => 'Success' ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

$app->post('/reservation/duplicate', function ($request, $response) use ($lrPluginUrl) {
    try {
        // Get all the request values
        $reservation_id = $request->getParsedBody()['reservation_id'];
        
        $token = (new Parser())->parse((string) $request->getAttribute('token'));         
        $usr = $token->getClaim('usr');

        // Create reservation
        $con = $this->apidb;
        $sql = "call spDuplicateReservation(:reservation_id, :employee_id, :store_id);"; 
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':reservation_id'           => $reservation_id,
            ':employee_id'              => $usr->id,
            ':store_id'                 => $usr->baseStoreId
        );
        $pre->execute( $values );
        $result = $pre->fetch();

        if( empty($result) ) {
            return $response->withJson(array('status' => false, 'error' => false, 'reservationId' => 0 ), 200);
        }

        return $response->withJson(array('status' => true, 'error' => false, 'reservationId' => $result['reservation_id'] ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*************************************
*************** ORDERS ***************
*************************************/

/* 
 * Get list of all the orders
 */
$app->get('/orders', function ($request, $response) use ($lrPluginUrl) {
    try {
        $con = $this->apidb;

        // orders
        $sql = "select *
                from vwOrder 
                order by orderDate desc 
                limit 200";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute();
        $orders = $pre->fetchall();

        if( !$orders )
            $orders = array();

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $orders ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * Get list of those orders that matches the identifier query
 * 
 * identifier:  varchar(15)                             // query string to look by customer name, customer phone, customer email,
 *                                                         order date and foreign order id
 */
$app->get('/orders/{identifier}', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;

        // orders
        $sql = "select orderId, foreignId, customerId, customerTypeId, customerType, customerFullName, customerPhone, customerEmail, orderDate, subtotal, otherChargesDiscount, tax, shipping, total, orderStatus, 'Order' category
                from vwOrder 
                where customerFullName like :identifier
                      or customerPhone like :identifier
                      or customerEmail like :identifier
                      or DATE_FORMAT(orderDate, '%Y-%m-%d') like :identifier
                      or orderId like :identifier
                      or foreignId like :identifier
                order by orderDate desc 
                limit 10";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => '%' . $request->getAttribute('identifier') . '%'
        );
        $pre->execute( $values );
        $orders = $pre->fetchall();

        if( !$orders )
            $orders = array();

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $orders ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * Create a new order
 * 
 * order: array(                                                                                            *
 *  order_id                varchar(10)                         // order id of the foreign system           *
 *  order_date              datetime                            // order date                               *
 *  customer:               array(
 *      foreign_id:             varchar(20)                     // customer id of the foreign system
 *      first_name:             varchar(50)             
 *      last_name:              varchar(50)
 *      phone:                  varchar(15)                                                                 *
 *      email:                  varchar(100)                                                        
 *      gender:                 varchar(1) - options:
 *                                  1: m
 *                                  2: f
 *                                  3: n
 *      address_line_1:         varchar(40)
 *      address_line_2:         varchar(40)
 *      address_city:           varchar(25)
 *      address_state:          varchar(2)
 *      address_zipcode:        varchar(10)
 *      measurements:			array(
 *			heigh_feet				tinyint				        // feet portion of the customer height
 *			height_inch				tinyint				        // inch portion of the customer height
 *			weight					decimal(10,2)		        // in pounds
 *			neck					decimal(10,2)		        // in inches
 *			shoulder				decimal(10,2)		        // in inches
 *			sleeve_length_left		decimal(10,2)		        // in inches
 *			sleeve_length_right		decimal(10,2)		        // in inches
 *			bicep					decimal(10,2)		        // in inches
 *			waist					decimal(10,2)		        // in inches
 *			wrist					decimal(10,2)		        // in inches
 *			chest 					decimal(10,2)		        // in inches
 *			stomach					decimal(10,2)		        // in inches
 *			jacket_length_front		decimal(10,2)		        // in inches
 *			jacket_length_back		decimal(10,2)		        // in inches
 *			hips					decimal(10,2)		        // in inches
 *			thigh					decimal(10,2)		        // in inches
 *			crotch					decimal(10,2)		        // in inches
 *			knee					decimal(10,2)		        // in inches
 *			leg_length				decimal(10,2)		        // in inches
 *			vest_length_front		decimal(10,2)		        // in inches
 *			vest_length_back		decimal(10,2)		        // in inches
 *			cuff					decimal(10,2)		        // in inches
 *			watch_hand				varchar(5) -options:        // hand in which the customer uses the watch
 *										left
 *										right
 *										none
 *			calf 					decimal(10,2)		        // in inches
 *			coat_sleeve_left		decimal(10,2)		        // in inches
 *			coat_sleeve_right		decimal(10,2)		        // in inches
 *		)
 *  )
 *  subtotal:               decimal(10,2)                       // order subtotal amount                    *
 *  other_charges_discount: decimal(10,2)                       // sum of other charges and discounts       *
 *  tax:                    decimal(10,2)                       // order tax amount                         *
 *  shipping:               decimal(10,2)                       // order shipping amount                    *
 *  total:                  decimal(10,2)                       // order total amount                       *
 *  status:                 varchar(15)                         // status id of the foreign system          *
 *  tracking_number:        varchar(50)                         // order tracking number
 *  carrier:                varchar(45)                         // shipping carrier
 *  pickup_date:            datetime                            // when the user pickup the outfit on store *
 *  referral_code:          varchar(10)                         // code that identify a referral entity
 *  employee_id:            int                                 // employee who made the order
 *  use_date:               date                                // on rental - date when outfit is going
 *                                                                 to be used
 *  want_date:              date                                // on rental - date when oufit is going
 *                                                                 to be pickedup at store
 *  return_date:            date                                // on rental - date when the client should
 *                                                                 return the outfit
 *  details:                array(                              // order details
 *      order_type              tinyint - options:              // which type of order is                   *
 *                                  1: Rent
 *                                  2: Sale
 *                                  3: Design to Measure
 *                                  4: Rent & Sale
 *      product_id:             int                             // product id                               *
 *      product_name:           varchar(45)                     // product name                             *
 *      qty:                    smallint                        // product quantity                         *
 *      price:                  decimal(10,2)                   // product price                            *
 *      total:                  decimal(10,2)                   // item total                               *
 *      image:                  mediumtext                      // on M2M Kiosk - design SVG
 *      is_couture:             tinyint(1) - options:           // on M2M Kiosk - whether item is a 
 *                                  0: no                          couture item or not
 *                                  1: yes
 *      event_participant_id:   int                             // on rental - id of an event participant
 *      modifications:          array(                          // on M2M Kiosk - item modification
 *          modification_type:      varchar(15) - options:      // modification type                        *
 *                                      modification
 *                                      fabric
 *          category_id:            smallint                    // category id                              *
 *          category_name:          varchar(50)                 // category name                            *
 *          option_id:              smallint                    // option id                                *
 *          option_name:            varchar(50)                 // option name                              *
 *          option_image:           varchar(250)                // option images                            *
 *          couture_description:    varchar(250)                // couture item description                 *
 *      )
 *  )
 *  charges_discounts:      array(                              // contains the details of the total of 
 *                                                                 the field other_charges_discount 
 *      other_charges_type:     tinyint - options                                                           *
 *                                  1: Discount
 *                                  2: Coupon
 *                                  3: Special Charges
 *                                  4: Damage Waiver
 *                                  5: Late Fee
 *      amount:                 decimal(10,2)                   // negative values for items that reduce    *
 *                                                                 the order total
 *      description:            varchar(250)                    // payment brief description
 *      code:                   varchar(30)                     // charge / discount code
 *  )
 *  payments:               array(                              // payments made to the order               
 *      date:                   datetime                                                                    *
 *      amount:                 decimal(10,2)                                                               *
 *      description:            varchar(250)
 *      method:                 varchar(20)                                                                 *
 *      confirmation_number:    varchar(25)                                                                 *
 *  )
 * )                   
 */

//Register a new order
$app->post('/order', function ($request, $response) use ($lrPluginUrl) {

    try {
        // Get the client information
        $client = $request->getAttribute('client');
        
        // Get all the request values
        $params = $request->getParsedBody()['order'];

        // return $response->withJson( array( 'status' => true, 'params' => $params, 'client' => $client ), 200 );

        // Check which client is making the request to make the changes
        // in the other platforms
        switch( $client['type'] ) {
            case 'Wintux':
                break;
            case 'POS':
                break;
            case 'LFA Hub':
                // Create order in the LFA Main DB
                $status = lfa_create_order( $params, $client, true, false, $this->apidb );
                break;
            case 'Web Store':
                // Create order in the LFA Main DB
                $status = lfa_create_order( $params, $client, true, false, $this->apidb );
                
                // Create order in the POS
                // pos_create_order( $params );

                // Create order in Wintux if it is a rental
                // wintux_create_order( $params );

                return $response->withJson( array( 'status' => true, 'message' => 'Success' ), 200 );
                break;
            case 'M2M Kiosk':
                // Create order in the LFA Main DB
                $status = lfa_create_order( $params, $client, true, false, $this->apidb );
                break;
            default:
                break;
        }
    }
    catch(\Exception $ex) {
        return $response->withJson(array( 'status' => false, 'error' => false, 'message' => $ex->getMessage()),422);
    }
})->add($mw_authorize);



/*
 * Create a rent order from a reservation
 * 
 * order:   array(
 *  reservation_id                   int                         // reservation id                                   *
 *  subtotal                         decimal(10,2)               // reservation subtotal amount                      *
 *  other_charges_discount_total     decimal(10,2)               // other charges and discount total amount          *
 *  tax                              decimal(10,2)               // tax amount                                       *
 *  shipping                         decimal(10,2)               // shipping or delivery charges                     *
 *  total                            decimal(10,2)               // order total amount                               *
 *  other_charges_discount: array(
 *      other_charges_type          tinyint                         // other charges type id
 *      description                 varchar(250)                    
 *      amount                      decimal(10,2)                   // if it is a discount or promo it must be negative
 *      code                        varchar(30)                     // if it is a promo code
 *  )                                
 * )
 */
$app->post('/order/rent', function ($request, $response) use ($lrPluginUrl) {
    try {
        // get making the request
        $client = $request->getAttribute('client');
        $token = (new Parser())->parse((string) $request->getAttribute('token'));
        $usr = $token->getClaim('usr');

        $params = $request->getParsedBody()['order'];
        $con = $this->apidb;
        
        // validate input
        $errors = array();
        if( empty( $params['reservation_id'] ) )
            array_push( $errors, 'Reservation ID missing' );
        if( empty( $params['subtotal'] ) )
            array_push( $errors, 'Subtotal missing' );
        if( $params['other_charges_discount_total'] == null ) 
            array_push( $errors, 'Other charges and discount total missing' );

        if( empty( $params['tax'] ) ) 
            array_push( $errors, 'Tax amount missing' );

        if( empty( $params['shipping'] ) ) 
            array_push( $errors, 'Shipping amount missing' );

        if( empty( $params['total'] ) ) 
            array_push( $errors, 'Total amount missing' );

        if( count($errors) > 0 ) 
            return $response->withJson(array('status' => false, 'error' => true, 'errors' => $errors ), 200);

        // Get reservation details
        $sql = "select customer_id, customer_measurement_id, event_id, event_role_id, use_date, want_date, return_date, pickup_location_id, comments, order_id
                from tblReservation 
                where id = :reservation_id;";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':reservation_id' => $params['reservation_id']
        );
        $pre->execute( $values );
        $reservation = $pre->fetch();

        if( $reservation['order_id'] > 0 ) {
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'An order already exist related to this reservation.', 'orderId' => $reservation['order_id']), 200);
        }

        // Create order
        $sql = "insert into tblOrder (
                                    order_date,
                                    customer_id,
                                    customer_measurement_id,
                                    subtotal,
                                    other_charges_discount,
                                    tax,
                                    shipping,
                                    total,
                                    status_id,
                                    store_id,
                                    employee_id,
                                    use_date,
                                    want_date,
                                    return_date,
                                    pickup_location_id,
                                    comments
                                    ) values (
                                    now(),
                                    :customer_id,
                                    :customer_measurement_id,
                                    :subtotal,
                                    :other_charges_discount,
                                    :tax,
                                    :shipping,
                                    :total,
                                    :status_id,
                                    :store_id,
                                    :employee_id,
                                    :use_date,
                                    :want_date,
                                    :return_date,
                                    :pickup_location_id,
                                    :comments)";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':customer_id'              => $reservation['customer_id'],
            ':customer_measurement_id'  => $reservation['customer_measurement_id'],
            ':subtotal'                 => $params['subtotal'],
            ':other_charges_discount'   => $params['other_charges_discount_total'],
            ':tax'                      => $params['tax'],
            ':shipping'                 => $params['shipping'],
            ':total'                    => $params['total'],
            ':status_id'                => 15,                  // status default es 1: pending payment
            ':store_id'                 => $usr->baseStoreId,   
            ':employee_id'              => $usr->id,
            ':use_date'                 => $reservation['use_date'],
            ':want_date'                => $reservation['want_date'],
            ':return_date'              => $reservation['return_date'],
            ':pickup_location_id'       => $reservation['pickup_location_id'],
            ':comments'                 => $reservation['comments'],
        );
        $pre->execute( $values );

        // Get order id
        $sql = "SELECT LAST_INSERT_ID() id;"; 
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute();
        $order_id = $pre->fetch()['id'];

        // update reservation to include the new order id
        $sql = "update tblReservation set order_id = :order_id 
                where id = :reservation_id;"; 
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':order_id'         => $order_id,
            ':reservation_id'   => $params['reservation_id']
        );
        $pre->execute( $values );

        // add items to the order
        $sql = "select r.product_id, p.style, p.category_id, p.description, p.price, p.sale_price
                from tblReservationItems r, tblProduct p
                where r.product_id = p.id and reservation_id = :reservation_id;";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':reservation_id' => $params['reservation_id']
        );
        $pre->execute( $values );
        $items = $pre->fetchall();

        foreach( $items as $item ) {
            $sql = "insert into tblOrderDetail (
                                            order_id,
                                            order_type_id,
                                            product_id,
                                            product_name, 
                                            qty,
                                            price,
                                            total,
                                            event_participant_id
                                            ) values (
                                            :order_id,
                                            :order_type_id,
                                            :product_id,
                                            :product_name,
                                            :qty,
                                            :price,
                                            :total,
                                            :event_participant_id);";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':order_id'             => $order_id,
                ':order_type_id'        => 1, // rent order type id is 1
                ':product_id'           => $item['product_id'],
                ':product_name'         => $item['description'],
                ':qty'                  => 1,
                ':price'                => $item['price'],
                ':total'                => $item['price'],
                ':event_participant_id' => null // cambiar esto por el id del participante del evento
            );
            $pre->execute( $values );
        }

        // add other charges and discounts to the order
        foreach( $params['other_charges_discount'] as $item ) {
            $sql = "insert into tblOrderOtherCharges (
                                            order_id,
                                            other_charges_type,
                                            amount,
                                            memo, 
                                            code
                                            ) values (
                                            :order_id,
                                            :other_charges_type,
                                            :amount,
                                            :memo,
                                            :code);";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':order_id'             => $order_id,
                ':other_charges_type'   => $item['other_charges_type'],
                ':amount'               => $item['amount'],
                ':memo'                 => $item['description'],
                ':code'                 => $item['code']
            );
            $pre->execute( $values );
        }

        // post reservation to wintux
        try {
            $sql = "select firstName, lastName, fullName, birthdate, phone, email, gender, addressLine1, addressLine2, addressCity, addressState, addressZipcode
                    from vwCustomer
                    where customerId = :customer_id;";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':customer_id'  => $reservation['customer_id']
            );
            $pre->execute( $values );
            $customer = $pre->fetch();

            $sql = "select coat_size, coat_length, coat_sleeve, shirt_size, pant_waist, pant_outseam, shoe_size
                    from tblCustomerRentalMeasurements
                    where id = :measurement_id;";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':measurement_id'  => $reservation['customer_measurement_id']
            );
            $pre->execute( $values );
            $measurements = $pre->fetch();

            $order = array(
                'wintux_store_id'           => $client['wintuxStoreId'],
                'customer'                  => array(
                    'id'                => $reservation['customer_id'],
                    'first_name'        => $customer['firstName'],
                    'last_name'         => $customer['lastName'],
                    'full_name'         => $customer['fullName'],
                    'birthdate'         => $customer['birthdate'],
                    'phone'             => $customer['phone'],
                    'email'             => $customer['email'],
                    'gender'            => $customer['gender'],
                    'address_line1'     => $customer['addressLine1'],
                    'address_line2'     => $customer['addressLine2'],
                    'address_city'      => $customer['addressCity'],
                    'address_state'     => $customer['addressState'],
                    'address_zipcode'   => $customer['addressZipcode'],
                    'measurements'      => array(
                        'id'            => $reservation['customer_measurement_id'],
                        'coat_size'     => $measurements['coat_size'],
                        'coat_length'   => $measurements['coat_length'],
                        'coat_sleeve'   => $measurements['coat_sleeve'],
                        'shirt_size'    => $measurements['shirt_size'],
                        'pant_waist'    => $measurements['pant_waist'],
                        'pant_outseam'  => $measurements['pant_outseam'],
                        'shoe_size'     => $measurements['shoe_size']
                    )
                ),
                'event_id'                  => $reservation['event_id'], 
                'event_role_id'             => $reservation['event_role_id'], 
                'use_date'                  => $reservation['use_date'], 
                'want_date'                 => $reservation['want_date'], 
                'return_date'               => $reservation['return_date'], 
                'pickup_location_id'        => $reservation['pickup_location_id'], 
                'comments'                  => $reservation['comments'], 
                'subtotal'                  => $params['subtotal'],
                'other_charges_discount'    => $params['other_charges_discount_total'],
                'tax'                       => $params['tax'],
                'shipping'                  => $params['shipping'],
                'total'                     => $params['total'],
                'status_id'                 => 0,                  // status default en wintux es 0: Not Printed
                'store_id'                  => $usr->baseStoreId,   
                'employee_id'               => $usr->id,
                'id'                        => $order_id,
                'date'                      => date('Y-m-d'),
                'items'                     => $items,
                'promos'                    => $params['other_charges_discount']
            );

            $wintux = new Wintux('pr', false, true);
            $foreign_id = $wintux->create_reservation( $order, $usr );

            $sql = "update tblOrder set foreign_id = :foreign_id
                    where id = :order_id;";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':order_id'             => $order_id,
                ':foreign_id'           => $foreign_id
            );
            $pre->execute( $values );

            // Write order on the POS
            // $rmh = new RMH( false, true );
            // $result = $rmh->create_order( $order, $usr );

            // return $response->withJson($result, 200);

            if( !$result['status'] && $result['error'] )
                return $response->withJson(array('status' => true, 'error' => true, 'orderId' => $order_id ), 200); // , 'message' => $result['message']
        }
        catch(\Exception $innerEx) {
            return $response->withJson(array('status' => true, 'error' => true, 'orderId' => $order_id, 'message' => $innerEx->getMessage() ), 200);
        }

        return $response->withJson(array('status' => true, 'error' => false, 'orderId' => $order_id), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

/*
 * Register a payment for a specific order
 * 
 * payment: array(                                                                                          *
 *  order_id:               varchar(20)                 // main order id or foreign order id                *
 *  order_id_type:          tinyint - options:          // type of the order id (internal or foreign)       *
 *                              1: LFA internal order id
 *                              2: Foreign order id 
 *  date:                   datetime                    // payment date                                     *
 *  amount:                 decimal(10,2)               // total amount paid                                *
 *  description:            varchar(250)                // payment brief description
 *  method:                 varchar(20)                 // payment method (e.g. Visa, MasterCard,           *
 *                                                         Cash, Debit, etc.)
 *  confirmation_number:    varchar(25)                 // payment confirmation number                      *
 * )
 */
$app->post('/order/payment', function ($request, $response) use ($lrPluginUrl) {
    try {
        $payment = $request->getParsedBody()['payment'];
        $con = $this->apidb;
        
        // Look for internal order id if the order id provided is a foreign id
        if ( $payment['order_id_type'] == 1 ) {
            $sql = "select id from tblOrder where id = :order_id";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':order_id' => $payment['order_id']
            );
            $pre->execute( $values );
            $order_id = $pre->fetch()['id'];

            if( empty( $order_id ) ) {
                return $response->withJson(array('status' => false,'message'=> 'There is no order with the given order id.'), 200);
            }
        }
        else if ( $payment['order_id_type'] == 2 ) {
            $sql = "select id from tblOrder where foreign_id = :foreign_id";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':foreign_id' => $payment['order_id']
            );
            $pre->execute( $values );
            $order_id = $pre->fetch()['id'];

            if( empty( $order_id ) ) {
                return $response->withJson(array('status' => false,'message'=> 'There is no order with the given foreign order id.'), 200);
            }

            $payment['order_id'] = $order_id;
        }

        $sql = "INSERT INTO `tblOrderPayment` (`order_id`,
                                                `payment_date`,
                                                `payment_amount`,
                                                `memo`,
                                                `payment_method`,
                                                `payment_confirmation_number`) VALUES (
                                                :order_id,
                                                :payment_date,
                                                :payment_amount,
                                                :memo,
                                                :payment_method,
                                                :payment_confirmation_number);";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':order_id' => $payment['order_id'],
            ':payment_date' => $payment['date'],
            ':payment_amount' => $payment['amount'],
            ':memo' => $payment['description'],
            ':payment_method' => $payment['method'],
            ':payment_confirmation_number' => $payment['confirmation_number']
        );
        $pre->execute( $values );

        return $response->withJson(array('status' => true, 'error' => false, 'message' => 'Success'), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

/*
 * Change order status
 * 
 * order_id:                varchar(20)                         // main order id or foreign order id                *
 * order_id_type:           tinyint - options:                  // type of the order id (internal or foreign)       *
 *                             1: LFA internal order id
 *                             2: Foreign order id 
 * order_status:            varchar(15)                         // status id of the foreign system                  *
 * order_status_type:       tinyint - options:                  // type of the order status (internal or foreign)       *
 *                             1: LFA internal status id
 *                             2: Foreign status id 
 */
$app->put('/order/status', function ($request, $response) use ($lrPluginUrl) {
    try {
        // Get the client information
        $client = $request->getAttribute('client');

        // Parameters
        $order_id = $request->getParsedBody()['order_id'];
        $order_id_type = $request->getParsedBody()['order_id_type'];
        $order_status = $request->getParsedBody()['order_status'];
        $order_status_type = $request->getParsedBody()['order_status_type'];

        // DB connection
        $con = $this->apidb;

        // Look for internal order id if the order id provided is a foreign id
        if ( $order_id_type == 1 ) {
            $sql = "select id from tblOrder where id = :order_id";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':order_id' => $order_id
            );
            $pre->execute( $values );
            $order_id = $pre->fetch()['id'];

            if( empty( $order_id ) ) {
                return $response->withJson(array('status' => false, 'error' => true, 'message'=> 'There is no order with the given order id.'), 200);
            }
        }
        else if ( $order_id_type == 2 ) {
            $sql = "select id from tblOrder where foreign_id = :foreign_id";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':foreign_id' => $order_id
            );
            $pre->execute( $values );
            $order_id = $pre->fetch()['id'];

            if( empty( $order_id ) ) {
                return $response->withJson(array('status' => false, 'error' => true, 'message'=> 'There is no order with the given foreign order id.'), 200);
            }
        }

        // Look for internal status id if the status id provided is a foreign id
        if ( $order_status_type == 1 ) {
            $sql = "select id from tblOrderStatus where id = :status_id and store_type_id = :store_type_id";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':status_id'        => $order_status,
                ':store_type_id'    => $client['client_type_id']
            );
            $pre->execute( $values );
            $order_status = $pre->fetch()['id'];

            if( empty( $order_status ) ) {
                return $response->withJson(array('status' => false, 'error' => true, 'message'=> 'There is no status with the given status id.'), 200);
            }
        }
        // transform order status
        else if ( $order_status_type == 2 ) {
            $sql = "select id from tblOrderStatus where foreign_id = :foreign_id and store_type_id = :store_type_id";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':foreign_id'       => $order_status,
                ':store_type_id'    => $client['client_type_id']
            );
            $pre->execute( $values );
            $order_status = $pre->fetch()['id'];

            if( empty( $order_status ) ) {
                return $response->withJson(array('status' => false, 'error' => true, 'message'=> 'There is no status with the given foreign status id.'), 200);
            }
        }

        $sql = "update tblOrder set status_id = :status_id where id = :order_id";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':status_id'    => $order_status,
            ':order_id'     => $order_id
        );
        $pre->execute( $values );

        if ( updateCustomer( $identifier, $customer_info, $this->apidb ) ) {
            return $response->withJson(array('status' => true, 'error' => false, 'message' => 'Success'), 200);  
        }
        else {
            return $response->withJson(array('status' => false, 'message' => 'Something went wrong. Please try again later.'), 200);
        }    
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

/*
 * Get list of those orders that matches the identifier query
 * 
 * identifier:  varchar(15)                             // query string to look by customer name, customer phone, customer email,
 *                                                         order date and foreign order id
 */
$app->get('/order/{order_id}', function ($request, $response) use ($lrPluginUrl) {
    try {
        $con = $this->apidb;

        // orders
        $sql = "call spOrder(:order_id)";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':order_id' => $request->getAttribute('order_id')
        );
        $pre->execute( $values );
        $order = $pre->fetch();

        if( !$orders )
            $orders = array();

        // Update order status from Wintux if it's a Wintux order
        if( !empty( $order['foreignId'] ) && $order['storeTypeId'] == 5 ) {
            $wintux = new Wintux( 'pr', false, true );
            $status = $wintux->get_order_status( $order['foreignId'] );
            return $response->withJson(array('status' => true, 'error' => false, 'data' => $status ), 200);

            if( !empty( $status ) ) {
                $order['orderStatus'] = $status['status'];
                $sql = "update tblOrder set status_id = (select id from tblOrderStatus where foreign_id = :status_id)
                        where id = :order_id";
                $pre1 = $this->apidb->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
                $values = array(
                    ':status_id'    => $status['status_id'],
                    ':order_id'     => $order['orderId']
                );
                $pre1->execute( $values );
            }
        }

        $pre->nextRowset();
        $items = $pre->fetchall();

        $order['items'] = $items;

        $pre->nextRowset();
        $customization = $pre->fetchall();

        $order['customization'] = $customization;

        $pre->nextRowset();
        $other_charges_discount = $pre->fetchall();

        $order['otherChargesDiscount'] = $other_charges_discount;

        $sql = "SELECT c.id, c.customer_type_id customerTypeId, ct.name customerType, c.first_name firstName, c.last_name lastName,
                        c.full_name fullName, c.birthdate, c.phone, c.email, c.gender, c.address_line_1 addressLine1, c.address_line_2 addressLine2,
                        c.address_city addressCity, c.address_state addressState, c.address_zipcode addressZipcode, c.date_created memberSince, floor(datediff(now(), c.date_created) / 365) memberYears,
                        (select count(1) from vwOrder o where o.storeTypeId = 2 and o.customerId = c.id) d2mOrders,
                        (select count(1) from vwOrder o where o.storeTypeId in (1, 5) and o.customerId = c.id) rentShopOrders
                FROM leonard5_lfa_main.tblCustomer c,
                    leonard5_lfa_main.tblCustomerType ct
                where c.customer_type_id = ct.id
                    and c.id = :customer_id";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':customer_id' => $order['customerId']
        );
        $pre->execute( $values );
        $customer = $pre->fetch();

        $order['customer'] = $customer;

        $sql = "select cm.id measurementId, cm.height_feet heightFeet, cm.height_inch heightInch, cm.weight, cm.neck,
                        cm.shoulder, cm.sleeve_length_left sleeveLengthLeft, cm.sleeve_length_right sleeveLengthRight, cm.bicep, cm.waist,
                        cm.wrist, cm.chest, cm.stomach, cm.jacket_length_front jacketLengthFront, cm.jacket_length_back jacketLengthBack,
                        cm.hips, cm.thigh, cm.crotch, cm.knee, cm.leg_length legLength, cm.vest_length_front vestLengthFront, 
                        cm.vest_length_back vestLengthBack, cm.cuff, cm.watch_hand watchHand, cm.calf, cm.coat_sleeve_left coatSleeveLeft,
                        cm.coat_sleeve_right coatSleeveRight, cm.date_created measurementDate
                from leonard5_lfa_main.tblCustomer c left outer join leonard5_lfa_main.tblCustomerMeasurements cm on c.id = cm.customer_id
                where c.id = :customer_id and cm.id = :measurement_id";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':customer_id'      => $order['customerId'],
            ':measurement_id'   => $order['customerMeasurementId']
        );
        $pre->execute( $values );
        $measurements = $pre->fetch();

        $order['customer']['measurements'] = empty( $measurements ) ? null : $measurements;

        $sql = "select cm.id measurementId, cm.coat_size coatSize, cm.coat_length coatLength, cm.coat_sleeve coatSleeve,
                        cm.shirt_size shirtSize, cm.pant_waist pantWaist, cm.pant_outseam pantOutseam, cm.shoe_size shoe, 
                        cm.date_created measurementDate
                from leonard5_lfa_main.tblCustomer c left outer join leonard5_lfa_main.tblCustomerRentalMeasurements cm on c.id = cm.customer_id
                where c.id = :customer_id and cm.id = :measurement_id";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':customer_id'      => $order['customerId'],
            ':measurement_id'   => $order['customerMeasurementId']
        );
        $pre->execute( $values );
        $rental_measurements = $pre->fetch();

        $order['customer']['rentalMeasurements'] = empty( $rental_measurements ) ? null : $rental_measurements;

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $order ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * Generate customer receipt of a rental order
 */
$app->get('/order/rent/receipt/{order_id}', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;
        $sql = "call spOrder(:order_id)";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':order_id' => $request->getAttribute('order_id')
        );
        $pre->execute( $values );
        $order = $pre->fetch();

        if( !$order )
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'Order not found.'), 200);

        $pre->nextRowset();
        $items = $pre->fetchall();

        $pre->nextRowset();
        $pre->nextRowset();
        $other_charges_discount = $pre->fetchall();

        $sql = "select coat_size coatSize, coat_length coatLength, coat_sleeve coatSleeve, shirt_size shirtSize, 
                       pant_waist pantWaist, pant_outseam pantOutseam, shoe_size shoeSize
                from tblCustomerRentalMeasurements
                where id = :measurement_id";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':measurement_id' => $order['customerMeasurementId']
        );
        $pre->execute( $values );
        $measurements = $pre->fetch();

        // Create PDF of the receipt
        require_once('tools/tcpdf_include.php');

        // create new PDF document
        $pageLayout = array(900.000, 216.000); //  or array($height, $width) 
        $pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, $pageLayout, true, 'UTF-8', false);

        // set document information
        $pdf->SetCreator(PDF_CREATOR);
        $pdf->SetAuthor('Leonardo Fifth Avenue');
        $pdf->SetTitle('Reservation #' . $order['foreignId']. ' Receipt');
        $pdf->SetSubject('Reservation Receipt');
        $pdf->SetPrintHeader(false);
        $pdf->SetPrintFooter(false);
        $pdf->IncludeJS('print(true);');

        $pdf->SetMargins(0, 0, 0);
        $pdf->SetHeaderMargin(0);
        $pdf->SetFooterMargin(0);

        // set auto page breaks
        $pdf->SetAutoPageBreak(TRUE, 0);

        $pdf->AddPage();

        // Create items table
        $items_html = '';
        foreach( $items as $item ) {
            $items_html .= '<tr>
                <td class="item" style="width:60%;">' . $item['category'] . '</td>
                <td class="item" style="width:20%;">' . $item['foreignId'] . '</td>';

            switch( $item['categoryId'] ) {
                case 'C':
                    $items_html .= '<td class="item" style="width:20%;">' . $measurements['coatSize'] . $measurements['coatLength'] . '</td>';
                    break;
                case 'P':
                    $wintux = new Wintux( 'pr', false, true );
                    $items_html .= '<td class="item" style="width:20%;">' . $wintux->get_wintux_pant_size( $measurements['pantWaist'] ) . '</td>';
                    break;
                case 'S':
                    $items_html .= '<td class="item" style="width:20%;">' . $measurements['shirtSize'] . '</td>';
                    break;
                case 'SH':
                    $items_html .= '<td class="item" style="width:20%;">' . $measurements['shoeSize'] . '</td>';
                    break;
                default:
                    $items_html .= '<td class="item" style="width:20%;">FA</td>';
                    break;
            }

            $items_html .= '</tr>';
        }

        // Create other charges and discounts table items
        $ocd_html = '';
        foreach( $other_charges_discount as $ocd ) {
            $ocd_html .= '<tr>
                    <td style="font-size:25px;text-align:right;width:47%;">' . ( $ocd['amount'] < 0 ? '- $' .  number_format(  $ocd['amount'] * -1.00, 2 ) : '$' . $ocd['amount'] ) . '</td>
                    <td style="width:3%;"></td>';

            if( $ocd['otherChargesTypeId'] == 6 )
                    $ocd_html .= '<td style="font-size:25px;width:50%;">' . $ocd['otherChargesType'] . ': ' . $ocd['code'] . '</td>';
            else if( $ocd['otherChargesTypeId'] == 7 )
                $ocd_html .= '<td style="font-size:25px;width:50%;">' . $ocd['otherChargesType'] . ': ' . $ocd['memo'] . '</td>';
            else 
                $ocd_html .= '<td style="font-size:25px;width:50%;">' . $ocd['otherChargesType'] .'</td>';

            $ocd_html .= '</tr>';
        }

        $html = '<style>
                * {
                    font-size: 30px;
                }
                .header {
                    width:100%;
                }
                .normal {
                    font-size: 30px;
                }
                .normal-numeric {
                    font-size: 30px;
                    text-align: right;
                }
                .logo {
                    width: 20%;
                }
                .title {
                    text-align:center;
                    margin-top: 20px;
                }
                .order-caption {
                    text-align: left;
                    font-size: 10px;
                }
                .item {
                    font-size: 31px;
                }
                .section {
                    border-top: 1px solid #000;
                    text-align: center;
                }
                .lowercase {
                    text-transform: lowercase;
                }
                .uppercase {
                    text-transform: uppercase;
                }
                .capitalize {
                    text-transform: capitalize;
                }
                .paragraph {
                    font-size: 22px;
                    text-align: justify;
                }
            </style>
            <table class="header" style="width:100%;">
                <tr>
                    <td colspan="2" style="text-align:center;">
                        <img src="tools/TCPDF/images/LFALogo.png" alt="LOGO" width="300" />
                    </td>
                </tr>
                <tr>
                    <td class="title" colspan="2"><p>' . $order['storePhone'] . '</p></td>
                </tr>
                <tr><td><br /></td></tr>
                <tr>
                    <td class="order-caption" style="width:60%;">
                        <table>
                            <tr><td style="font-size:25px;">Use Date: </td><td style="font-size:25px;">' . date( 'm/d/Y', strtotime( $order['useDate'] ) ) . '</td></tr>
                            <tr><td style="font-size:25px;">Pickup Date: </td><td style="font-size:25px;">' . date( 'm/d/Y @ h:i a', strtotime( $order['wantDate'] ) ) . '</td></tr>
                            <tr><td style="font-size:25px;">Return Date: </td><td style="font-size:25px;">' . date( 'm/d/Y', strtotime( $order['returnDate'] ) ) . '</td></tr>
                            <tr><td style="font-size:25px;">Pickup At: </td><td style="font-size:25px;">' . $order['pickupLocation'] . '</td></tr>
                            <tr><td style="font-size:25px;">Occasion: </td><td style="font-size:25px;">' . '' . '</td></tr>
                        </table>
                    </td>
                    <td style="width:40%;text-align:right;font-size:30px;"> 
                        <table>
                            <tr><td>ORDER #</td></tr>
                            <tr><td style="font-size:50px;"><b>' . $order['foreignId'] . '</b></td></tr>
                        </table>
                    </td>
                </tr>
            </table>
            <br/><br/>
            <table>
                <thead>
                    <tr>
                        <th style="width:60%;"><b>ITEM</b></th>
                        <th style="width:20%;"><b>STYLE</b></th>
                        <th style="width:20%;"><b>SIZE</b></th>
                    </tr>
                </thead>
                <tbody>
                    ' . $items_html . '
                </tbody>
            </table>
            
            <br />
            <hr />
            <br />
            <h4 class="uppercase" style="text-align:center;">Terminos y condiciones del contrato de alquiler</h4>

            <p class="paragraph">Leonardo Fifth Avenue (LFA) no asume responsabilidad y el cliente expresamente releva a LFA de 
            responsabilidad por cualquier daño, incidental y/o consiguiente a/o en relación con el alquiler, compra y/o uso de 
            mercancía. A cambio del pago de $10.00 por concepto de relevo de responsabilidad, LFA cede su derecho de reclamar al 
            cliente los daños no intencionales causados a la ropa alquilada. Este relevo de responsabilidad no incluye daños 
            intencionales o maliciosos causados a la ropa alquilada ni daños ocasionados, por perdida o robo El cliente 
            expresamente releva a LFA de responsabilidad por la perdida o daños ocasionados a artículos personales dejados en 
            la ropa. El cliente asume completa responsabilidad por los daños o pérdidas de artículos alquilados y se compromete 
            a realizar el pago para el remplazo del artículo. El cliente se compromete a devolver los artículos alquilados el 
            próximo día laborable luego de la fecha de uso arriba indicada y autoriza que se le imponga un cargo de $25.00 por 
            cada día de atraso en la devolución de la ropa alquilada. Toda orden de ropa alquilada que no sea devuelta dentro 
            de un termino de 7 días a partir a la fecha indicada de retorno, será considerada en delincuencia y se aplicará un 
            cargo de $600.00 por concepto de reemplazo de mercancía, el cual el cliente autoriza. Dicha deuda será liquida y 
            exigible al momento de la notificación de esta. Alquileres y sus respectivos cobros podrían ser anulables (VOID) 
            únicamente el mismo día de la transacción antes de las 5:00pm; esto no aplicará a ordenes “RUSH” con fecha de entrega 
            de 48 horas o menos. En cancelaciones no se harán reembolsos de dinero, únicamente se dará crédito de tienda. En 
            alquileres cancelados con dos semanas o más de anticipación a la fecha de uso se retendrán $40.00 por cargos de 
            servicio y la diferencia se otorgará con un crédito de tienda. En alquileres cancelados con menos de dos semanas de 
            anticipación a la fecha de uso no se aplicará devolución crédito de tienda. Cambios en ordenes realizados con menos 
            de 7 días calendarios de anticipación a la fecha de uso conlleva un cargo de $10.00. Estilos en oferta serán honrados 
            mientras duren; de no estar disponibles, serían sustituidos por otros similares de igual o superior calidad. Para 
            garantizar un servicio de excelencia y poder resolver cualquier situación de medidas imprecisas, mercancía con 
            defectos de fábrica, o en discrepancia con la orden, el cliente se compromete a recoger la orden el día antes del 
            día de uso (' . date( 'm/d/Y', strtotime( $order['wantDate'] ) ) . ' después de las ' . date( 'h:i a', strtotime( $order['wantDate'] ) ) . 
            '), verificar toda mercancía y medírsela en la tienda. De requerir algún cambio el 
            cliente deberá recoger el mismo al próximo día (día del evento). LFA no se hace responsable por errores en ordenes 
            recogidas el mismo día de uso, o con errores no reclamados al momento de recoger la mercancía en la tienda. El cliente 
            acepta que la responsabilidad legal de LFA en cuanto a cualquier asunto o reclamación relacionada con un alquiler o 
            compra de mercancía estará limitada a un máximo del total de cargos que el cliente haya pagado por el concepto del 
            alquiler o compra de la mercancía. Certifico haber leído, entendido y estar de acuerdo con los términos del presente 
            acuerdo de alquiler.</p>
            
            <br />
            <table>
                <tr>
                    <td style="width:10%;"></td>
                    <td style="width:80%;">
                        <table>
                            <tr>
                                <td style="font-size:25px;text-align:right;width:47%;">$' . $order['subtotal'] . '</td>
                                <td style="width:3%;"></td>
                                <td style="font-size:25px;width:50%;">Subtotal</td>
                            </tr>
                            ' . $ocd_html . '
                            <tr>
                                <td style="font-size:25px;text-align:right;width:47%;">$' . $order['shipping'] . '</td>
                                <td style="width:3%;"></td>
                                <td style="font-size:25px;width:50%;">Delivery</td>
                            </tr>
                            <tr>
                                <td style="border-bottom: 1px solid black;font-size:25px;text-align:right;width:47%;">$' . $order['tax'] . '</td>
                                <td style="width:3%;"></td>
                                <td style="font-size:25px;width:50%;">Tax</td>
                            </tr>
                            <tr>
                                <td style="font-size:25px;text-align:right;width:47%;">$' . $order['total'] . '</td>
                                <td style="width:3%;"></td>
                                <td style="font-size:25px;width:50%;">Total</td>
                            </tr>
                            <tr>
                                <td></td>
                                <td></td>
                                <td></td>
                            </tr>
                            <tr>
                                <td style="font-size:30px;text-align:right;width:47%;"><b>$' . $order['balance'] . '</b></td>
                                <td style="width:3%;"></td>
                                <td style="font-size:30px;width:50%;">Balance Due</td>
                            </tr>
                        </table>
                    </td>
                    <td style="width:10%;"></td>
                </tr>
            </table>

            <br /><br /><br /><br /><br /><br />

            <table>
                <tr>
                    <td><b>Customer</b></td>
                </tr>
                <tr>
                    <td style="font-size:30px;">' . $order['customerFullName'] . '</td>
                </tr>
                <tr>
                    <td style="font-size:30px;">' . $order['customerAddressLine1'] . '</td>
                </tr>
                ' . ( empty( $order['customerAddressLine2'] ) ? '' : '<tr><td style="font-size:30px;">' . $order['customerAddressLine2'] . '</td></tr>' ) . '
                <tr>
                    <td style="font-size:30px;">' . $order['customerAddressCity'] . ' ' . $order['customerAddressState'] . ', ' . $order['customerAddressZipcode'] . '</td>
                </tr>
            </table>

            <br /><br /><br /><br /><br /><br /><br /><br />

            <hr />
            <span style="font-size:30px;">Customer signature</span>';

        $pdf->writeHTML($html, true, false, true, false, '');

        // Close and output PDF document
        // This method has several options, check the source code documentation for more information.
        $pdf->Output('LFA_-_Rental_Order_' . $order['foreignId'] . '.pdf', 'D');
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*************************************
************** CUSTOMER **************
*************************************/

/*
 * Get customer list
 */
$app->get('/customers', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;
        $sql = "select customerId, customerTypeId, customerType, firstName, lastName, fullName, phone, email, gender, dateCreated 
                from vwCustomer 
                order by dateCreated desc 
                limit 200";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute();
        $customers = $pre->fetchall();

        if( !$customers )
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'There are no customer created.'), 200);

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $customers), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * Get list of those customer that matches the identifier query
 * 
 * identifier:  varchar(15)                             // query string that represents part of the user phone number, name or email
 */
$app->get('/customers/{identifier}', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;
        $sql = "select c.customerId, customerTypeId, customerType, firstName, lastName, fullName, phone, email, gender, dateCreated, addressLine1, addressLine2, addressCity, addressState, addressZipcode, ck.memberYears, ck.d2mOrders, ck.rentShopOrders, 'Customer' category
                from vwCustomer c, vwCustomerKeyItems ck
                where c.customerId = ck.customerId
                      and (phone like :identifier
                      or email like :identifier
                      or fullName like :identifier)
                order by dateCreated desc 
                limit 10";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => '%' . $request->getAttribute('identifier') . '%'
        );
        $pre->execute( $values );
        $customers = $pre->fetchall();

        if( !$customers )
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'There are no customer for the given identifier.'), 200);

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $customers), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * Create a new customer
 * 
 * customer: array(                                                                                 *
 *  type:                   smallint - options:         // type of customer                         *
 *                                  1: Person, 
 *                                  2: Company,
 *  first_name:             varchar(50)                 // first name                               *
 *  last_name:              varhcar(50)                 // last name                                *
 *  full_name:              varchar(101)                // full name - only required when           *
 *                                                         first name and last name are not 
 *                                                         provided
 *  phone:                  varchar(15)                 // customer phone number                    *
 *  email:                  varchar(100)                // customer email address                   
 *  gender:                 varchar(1) - options:       // customer gender                          *
 *                              m: Male, 
 *                              f: Female, 
 *                              n: neutral
 *  address_line_1:         varchar(40)                 // customer address line 1
 *  address_line_2:         varchar(40)                 // customer address line 2
 *  address_city:           varchar(25)                 // customer address city
 *  address_state:          varchar(2)                  // customer address state
 *  address_zipcode:        varchar(10)                 // customer address zip code
 * )                   
 */
$app->post('/customer', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;
        $params = $request->getParsedBody()['customer'];

        $customer_info = (object)array(
            'customerTypeId'        => $params['type'],
            'customerName'          => $params['first_name'],
            'customerLastName'      => $params['last_name'],
            'customerFullName'      => $params['full_name'],
            'customerBirthdate'     => $params['birthdate'],
            'customerPhone'         => $params['phone'],
            'customerEmail'         => $params['email'],
            'customerGender'        => $params['gender'],
            'customerAddressLine1'  => $params['address_line_1'],
            'customerAddressLine2'  => $params['address_line_2'],
            'customerCity'          => $params['address_city'],
            'customerState'         => $params['address_state'],
            'customerZipcode'       => $params['address_zipcode']
        );

        // check if customer with same email or phone already exist
        $sql = "select id customerId from tblCustomer where phone = :phone or email = :email";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':phone' => $customer_info->customerPhone,
            ':email' => $customer_info->customerEmail
        );
        $pre->execute( $values );
        $result = $pre->fetch();

        if( !empty( $result ) )
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'Customer already exist.', 'customerId' => $result['customerId']), 200);

        $customer_id = createCustomer( $customer_info, $con );

        if( empty( $customer_id ) ) 
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'Something went wrong. Please try again later.'), 200);

        return $response->withJson(array('status' => true, 'message' => 'Success', 'customerId' => $customer_id), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * Add measurements to a customer
 * 
 * identifier:  varchar(15)                             // Can be the LFA main db id or the 
 *                                                         phone number
 * measurements: array(
 *  height_feet:            tinyint - options:          // feet portion of the customer height
 *  height_inch:            tinyint                     // inches portion of the customer height
 *  weight:                 decimal(10,2)               // weight in pounds (lbs)
 *  neck:                   decimal(10,2)               // neck measurement (in)
 *  shoulder:               decimal(10,2)               // shoulder measurement (in)
 *  sleeve_length_left:     decimal(10,2)               // left sleeve length measurement (in)
 *  sleeve_length_right:    decimal(10,2)               // right sleeve length measurement (in)
 *  bicep:                  decimal(10,2)               // bicep measurement (in)
 *  waist:                  decimal(10,2)               // waist measurement (in)
 *  wrist:                  decimal(10,2)               // wrist measurement (in)
 *  chest:                  decimal(10,2)               // chest measurement (in)
 *  stomach:                decimal(10,2)               // stomach measurement (in)
 *  jacket_length_front:    decimal(10,2)               // front jacket length measurement (in)
 *  jacket_length_back:     decimal(10,2)               // back jacket length measurement (in)
 *  hips:                   decimal(10,2)               // hips measurement (in)
 *  thigh:                  decimal(10,2)               // thigh measurement (in)
 *  crotch:                 decimal(10,2)               // crotch measurement (in)
 *  knee:                   decimal(10,2)               // knee measurement (in)
 *  leg_length:             decimal(10,2)               // leg length measurement (in)
 *  vest_length_front:      decimal(10,2)               // front vest length measurement (in)
 *  vest_length_back:       decimal(10,2)               // back vest length measurement (in)
 *  cuff:                   decimal(10,2)               // cuff measurement (in)
 *  watch_hand:             varchar(5)                  // watch hand - options:
 *                                                              left,
 *                                                              right,
 *                                                              none
 *  calf:                   decimal(10,2)               // calf measurement (in)
 *  coat_sleeve_left:       decimal(10,2)               // left coat sleeve measurement (in)
 *  coat_sleeve_right:      decimal(10,2)               // right coat sleeve measurement (in)
 * )                   
 */
$app->post('/customer/measurement', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;
        $customer_id = $request->getParsedBody()['identifier'];
        $measurements = $request->getParsedBody()['measurements'];

        // Check if the customer id is valid and a customer exists with the id
        if ( empty( $customer_id ) )
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'The customer identifier is requried.'), 200);

        if( empty( $measurements['height_feet'] ) )
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'You must provide a value for the customer height (feet).'), 200);
        if( intval( $measurements['height_inch'] ) < 0 || intval( $measurements['height_inch'] ) > 11 )
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'You must provide a valid value for the customer height (inch).'), 200);
        if( empty( $measurements['weight'] ) )
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'You must provide a value for the customer weight (lbs).'), 200);
        
        $sql = "select id from tblCustomer where id = :customer_id or phone = :customer_id";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':customer_id' => $customer_id
        );
        $pre->execute( $values );
        $result = $pre->fetch()['id'];

        if( empty( $result ) ) 
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'There are no customer for the given customer identifier.'), 200);

        saveCustomerMeasurements( (object)$measurements, $customer_id, $con );        

        return $response->withJson(array('status' => true, 'error' => false, 'message' => 'Success'), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

/*
 * Add measurements to a customer
 * 
 * identifier:  varchar(15)                      // Can be the LFA main db id or the 
 *                                                  phone number
 * measurements: array(
 *  coat_size:       tinyint                     // feet portion of the customer height
 *  coat_length:     varchar(1)                  // inches portion of the customer height
 *  coat_sleeve:     decimal(10,2)               // weight in pounds (lbs)
 *  waist:           varchar(5)                  // neck measurement (in)
 *  outseam:         decimal(10,2)               // shoulder measurement (in)
 *  shirt_size:      varchar(5)                  // left sleeve length measurement (in)
 *  shoe_size:       decimal(10,2)               // right sleeve length measurement (in)
 * )                   
 */
$app->post('/customer/rental-measurement', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;
        $customer_id = $request->getParsedBody()['identifier'];
        $measurements = $request->getParsedBody()['measurements'];

        // Check if the customer id is valid and a customer exists with the id
        if ( empty( $customer_id ) )
            return $response->withJson(array('status' => false, 'message' => 'The customer identifier is requried.'), 200);

        $sql = "select id from tblCustomer where id = :customer_id or phone = :customer_id";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':customer_id' => $customer_id
        );
        $pre->execute( $values );
        $result = $pre->fetch()['id'];

        if( empty( $result ) ) 
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'There are no customer for the given customer identifier.'), 200);

        saveCustomerRentalMeasurements( (object)$measurements, $customer_id, $con );

        $sql = "select id measurementId, coat_size coatSize, coat_length coatLength, coat_sleeve coatSleeve,
                       shirt_size shirtSize, pant_waist pantWaist, pant_outseam pantOutseam,
                       shoe_size shoe, date_created measurementDate
                from tblCustomerRentalMeasurements 
                where customer_id = :customer_id 
                order by date_created desc 
                limit 1";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':customer_id' => $customer_id
        );
        $pre->execute( $values );
        $result = $pre->fetch();

        return $response->withJson(array('status' => true, 'error' => false, 'message' => 'Success', 'measurements' => $result), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

/*
 * Get customer information including latests measurements
 * 
 * identifier: varchar(15)                   // Can be the LFA main db id or the phone number
 */
$app->get('/customer/{identifier}', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;
        $sql = "call spCustomerInformation(:identifier)";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => $request->getAttribute('identifier')
        );
        $pre->execute( $values );
        $customer = $pre->fetch();

        if( !$customer ) 
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'Customer doesn\'t exist'), 200);

        $pre->nextRowset();
        $measurements = $pre->fetch();

        $customer['measurements'] = $measurements;

        $pre->nextRowset();
        $rental_measurements = $pre->fetch();

        $customer['rentalMeasurements'] = $rental_measurements;

        $pre->nextRowset();
        $orders = $pre->fetchall();

        $customer['orders'] = $orders;

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $customer), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

/*
 * Update customer information
 * 
 * identifier: varchar(15)                   // Can be the LFA main db id or the phone number
 */
$app->put('/customer/{identifier}', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $params = $request->getParsedBody()['customer'];
        $identifier = $request->getAttributes('identifier');

        $customer_info = (object)array(
            'customerTypeId'        => $params['type'],
            'customerName'          => $params['first_name'],
            'customerLastName'      => $params['last_name'],
            'customerFullName'      => $params['full_name'],
            'customerBirthdate'     => $params['birthdate'],
            'customerPhone'         => $params['phone'],
            'customerEmail'         => $params['email'],
            'customerGender'        => $params['gender'],
            'customerAddressLine1'  => $params['address_line_1'],
            'customerAddressLine2'  => $params['address_line_2'],
            'customerCity'          => $params['address_city'],
            'customerState'         => $params['address_state'],
            'customerZipcode'       => $params['address_zipcode']
        );

        if ( updateCustomer( $identifier, $customer_info, $this->apidb ) ) {
            return $response->withJson(array('status' => true,'message' => 'Success'), 200);  
        }
        else {
            return $response->withJson(array('status' => false, 'message' => 'Something went wrong. Please try again later.'), 200);
        }    
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

/*
 * Get latest customer measurements
 * 
 * identifier: varchar(15)              // customer id or customer phone number
 */
$app->get('/customer/measurements/{identifier}', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;
        $sql = "call spCustomerMeasurement(:identifier)";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => $request->getAttribute('identifier')
        );
        $pre->execute( $values );
        $measurements = $pre->fetch();

        if( empty( $measurements ) ) {
            return $response->withJson(array('status' => false, 'error' => false, 'message'=> 'Measurements not found for the given user.' ), 200);
        }

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $measurements), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

/*
 * Get latest customer orders
 * 
 * identifier: varchar(15)              // customer id or customer phone number
 */
$app->get('/customer/orders/{identifier}', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;
        $sql = "call spCustomerOrders(:identifier)";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => $request->getAttribute('identifier')
        );
        $pre->execute( $values );
        $orders = $pre->fetchall();

        $pre->nextRowset();
        $order_details = $pre->fetchall();

        $pre->nextRowset();
        $order_details_customization = $pre->fetchall();

        $pre->nextRowset();
        $other_charges = $pre->fetchall();

        foreach ( $orders as $order ) {
            
            // Other charges
            $order['otherCharges'] = array();
            foreach( $other_charges as $oc ) {

                if( $order['orderId'] == $oc['orderId'] ) {
                    array_push( $order['otherCharges'], $oc );
                }

            }

            // Order detail
            $order['detail'] = array();
            foreach( $order_details as $od ) {

                if( $order['orderId'] == $od['orderId'] ) {

                    // Order detail customization
                    $od['customization'] = array();
                    if ( $order['orderTypeId'] == 3 ) { // Design to measure order
                        foreach( $order_details_customization as $odc ) {

                            if( $od['itemId'] == $oc['itemId'] ) {
                                array_push( $od['customization'], $odc );
                            }

                        }
                    }

                    array_push( $order['detail'], $od );
                }

            }

        }

        return $response->withJson(array('status' => true, 'error' => false, 'data'=> $orders), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

/*************************************
*************** EVENTS ***************
*************************************/

/*
 * Get list of those events that matches the identifier query
 * 
 * identifier:  varchar(15)                             // query string that represents part of the event id, coordinator phone, 
 *                                                         coordinator name, location or date
 */
$app->get('/events', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;
        $sql = "select *
                from vwEvent
                order by eventDate desc";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute( );
        $events = $pre->fetchall();

        if( !$events )
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'No events found.'), 200);

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $events), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * Get list of those events that matches the identifier query
 * 
 * identifier:  varchar(15)                             // query string that represents part of the event id, coordinator phone, 
 *                                                         coordinator name, location or date
 */
$app->get('/events/{identifier}', function ($request, $response) use ($lrPluginUrl) {
    try {
        
        $con = $this->apidb;
        $sql = "select eventId, occasion, eventType, location, coordinator1FullName, coordinator1Phone, coordinator1Email, eventDate, status, 'Event' category
                from vwEvent 
                where eventId like :identifier
                      or wintuxEventId like :identifier 
                      or coordinator1FullName like :identifier 
                      or coordinator1Phone like :identifier
                      or coordinator2FullName like :identifier 
                      or coordinator2Phone like :identifier
                      or location like :identifier
                      or eventDate like :identifier
                order by eventDate desc 
                limit 10";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':identifier' => '%' . $request->getAttribute('identifier') . '%'
        );
        $pre->execute( $values );
        $events = $pre->fetchall();

        if( !$events )
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'There are no events for the given identifier.'), 200);

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $events), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * Get event information
 * 
 * event_code: int
 */
$app->get('/event/{event_code}', function ($request, $response) use ($lrPluginUrl) {
    try {
        $con = $this->apidb;
        $sql = "call spEvent(:event_code)";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':event_code' => $request->getAttribute('event_code')
        );
        $pre->execute( $values );
        $result = $pre->fetch();

        if(!$result)
            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'Event not found'), 200);

        $pre->nextRowset();
        $result['participants'] = $pre->fetchall();

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $result), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('status' => false, 'error' => true, 'message' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * To create an event the endpoint need the following information
 * 
 * foreign_key: varchar(10)                                 // If do not apply enter empty string
 * event: array(
 *  main_contact_name:      varchar(100)                    // main contact of the event        *
 *  main_contact_phone:     varchar(15)                     // main contact phone number        *
 *  main_contact_email:     varchar(100)                    // main contact email               *
 *  occasion:               varchar(50)                     // event name                       *
 *  event_type:             tinyint - options               // event type                       *
 *                              1: Wedding, 
 *                              2: Prom, 
 *                              3: Quinceañera, 
 *                              4: Gala, 
 *                              5: Other
 *  location:               varchar(75)                     // even location                    *
 *  event_date:             date                            // event date                       *
 *  discount_code:          varchar(45)                     // discount code
 *  employee_id:            int,                            // id of the employee that helped   *
 *  referral_source:        smallint - options:             // how the customer hear about us
 *                              1: Billboard, 
 *                              2: Bridal Show, 
 *                              3: Competitors Coupon, 
 *                              4: Direct Mail Coupon, 
 *                              5: Email Blast,
 *                              6: Internet / Website,
 *                              7: Newspaper,
 *                              8: Referred By Friend,
 *                              9: Radio,
 *                              10: Television,
 *                              11: VIP Card,
 *                              12: Walk-in,
 *                              13: Telemarketing,
 *                              14: Facebook,
 *                              15: Instagram,
 *                              16: Wedding Coordinator,
 *  guests:                 array(                          // guests information               *
 *      customer_id:            int - null if is new        // id of already created customer
 *      type:                   smallint - options:         // type of customer                 
 *                                  1: Person, 
 *                                  2: Company,
 *      first_name:             varchar(50)                 // first name
 *      last_name:              varhcar(50)                 // last name
 *      full_name:              varchar(101)                // full name - only required when 
 *                                                             first name and last name are not 
 *                                                             provided
 *      phone:                  varchar(15)                 // customer phone number
 *      email:                  varchar(100)                // customer email address
 *      gender:                 varchar(1) - options:       // customer gender
 *                                  m: Male, 
 *                                  f: Female, 
 *                                  n: neutral
 *      address_line_1:         varchar(40)                 // customer address line 1
 *      address_line_2:         varchar(40)                 // customer address line 2
 *      address_city:           varchar(25)                 // customer address city
 *      address_state:          varchar(2)                  // customer address state
 *      address_zipcode:        varchar(10)                 // customer address zip code
 *      event_role              smallint - options:         // guest role on the event
 *                                  1: Groom
 *                                  2: Gala
 *                                  3: Wedding Guest
 *                                  4: Aniversary
 *                                  5: Ship
 *                                  6: Birthday
 *                                  7: Quinceañero
 *                                  8: Comitiva
 *                                  9: Other
 *                                  10: Student
 *                                  11: Best Man
 *                                  12: Family Member
 *                                  13: Groomsman
 *                                  14: Usher
 *      coat_style:             varchar(10)                 // coat style sku / upc
 *      pant_style:             varchar(10)                 // pant style sku / upc
 *      shirt_style:            varchar(10)                 // shirt style sku / upc
 *      jewelry_style:          varchar(10)                 // jewelry style sku / upc
 *      accessory_style:        varchar(10)                 // accessory style sku / upc
 *      neckwear_style:         varchar(10)                 // neckwear style sku / upc
 *      shoe_style:             varchar(10)                 // shoe style sku / upc
 *      other_style:            varchar(10)                 // other style sku / upc
 *      other_2_style:          varchar(10)                 // other 2 style sku / upc
 *      other_3_style:          varchar(10)                 // other 3 style sku / upc
 *      webstore_product:       varchar(20)                 // website product id | product variation
 *  )
 * )
 */
$app->post('/event', function ($request, $response) use ($lrPluginUrl) {
    try {
        // Get all the request values
        $params = $request->getParsedBody()['event'];
        $foreign_key = $request->getParsedBody()['foreign_key'];

        $token = (new Parser())->parse((string) $request->getAttribute('token'));         
        $usr = $token->getClaim('usr');

        $con = $this->apidb;
        
        // check if foreign key already exist if it isn't empty
        if( !empty( $foreign_key ) ) {
            $sql = "select id from tblEvent where foreign_key = :foreign_key;";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':foreign_key' => $foreign_key
            );
            $pre->execute( $values );
            if ( !empty( $pre->fetch()['id'] ) ) 
                return $response->withJson(array('status' => false,'message'=> 'Event foreign key already associated with another event.'), 200);
        }

        // get event status
        $sql = "select id from tblEventStatus where wintux_status_id = :status_id";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':status_id' => '25'
        );
        $pre->execute( $values );
        $event_status_id = $pre->fetch()['id'];

        // Create event
        $sql = "INSERT INTO `tblEvent` (`foreign_key`,
                                        `coordinator1_customer_id`,
                                        `coordinator1_comment`,
                                        `coordinator2_customer_id`, 
                                        `coordinator2_comment`,
                                        `occasion`,
                                        `event_type_id`,
                                        `location`,
                                        `event_date`,
                                        `status_id`,
                                        `date_created`,
                                        `discount_code`,
                                        `origin_store_id`,
                                        `employee_id`,
                                        `referral_source_id`,
                                        `comment`,
                                        `default_style`) VALUES (
                                        :wintux_event_code,
                                        :coordinator1_customer_id,
                                        :coordinator1_comment,
                                        :coordinator2_customer_id,
                                        :coordinator2_comment,
                                        :occasion,
                                        :event_type_id,
                                        :location,
                                        :event_date,
                                        :status_id,
                                        NOW(),
                                        :discount_code,
                                        :origin_store_id,
                                        :employee_id,
                                        :referral_source_id,
                                        :comment,
                                        :default_style);";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':wintux_event_code'            => empty($foreign_key) ? null : $foreign_key,
            ':coordinator1_customer_id'     => $params['organizers'][0]['customerId'],
            ':coordinator1_comment'         => $params['organizers'][0]['comment'],
            ':coordinator2_customer_id'     => empty( $params['organizers'][1]['customerId'] ) ? null : $params['organizers'][1]['customerId'],
            ':coordinator2_comment'         => empty( $params['organizers'][1]['comment'] ) ? null : $params['organizers'][1]['comment'],
            ':occasion'                     => $params['occasion'],
            ':event_type_id'                => $params['event_type'],
            ':location'                     => $params['location'],
            ':event_date'                   => $params['event_date'],
            ':status_id'                    => $event_status_id,
            ':discount_code'                => $params['promo'], 
            ':origin_store_id'              => $usr->baseStoreId,
            ':employee_id'                  => $usr->id, 
            ':referral_source_id'           => $params['referral_source'],
            ':comment'                      => $params['comment'],
            ':default_style'                => $params['default_style']
        );
        $pre->execute( $values );

        // Get event id
        $sql = "SELECT LAST_INSERT_ID() event_id;";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute();
        $event_id = $pre->fetch()['event_id'];

        // Create event participants
        foreach( $params['guests'] as $guest ) {
            // First check if their is a customer with the given phone number or email
            $customer_id = $guest['id'];
            if ( empty( $customer_id ) ) {
                $sql = "select customerId from vwCustomer where phone = :phone or email = :email";
                $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
                $values = array(
                    ':phone' => $guest['phone'],
                    ':email' => $guest['email']
                );
                $pre->execute( $values );
                $customer_id = $pre->fetch()['customerId'];
            }

            // Create customer if it doesn't exist
            if ( empty( $customer_id ) ) {

                $customer_info = (object)array(
                    'customerTypeId'        => 1,
                    'customerName'          => $guest['first_name'],
                    'customerLastName'      => $guest['last_name'],
                    'customerFullName'      => $guest['full_name'],
                    'customerPhone'         => $guest['phone'],
                    'customerEmail'         => $guest['email'],
                    'customerGender'        => $guest['gender'],
                    'customerAddressLine1'  => $guest['address_line_1'],
                    'customerAddressLine2'  => $guest['address_line_2'],
                    'customerCity'          => $guest['address_city'],
                    'customerState'         => $guest['address_state'],
                    'customerZipcode'       => $guest['address_zipcode']
                );
                
                $customer_id = createCustomer( $customer_info, $con );
            }

            // Get guest style information
            $coat_style = null;
            $pant_style = null;
            $shirt_style = null;
            $jewelry_style = null;
            $accessory_style = null;
            $neckwear_style = null;
            $shoe_style = null;
            $other_style = null;
            $other_2_style = null;
            $other_3_style = null;
            if( !empty( $guest['style'] ) ) {
                $sql = 'select id, style, category_id
                        from tblProduct
                        where id in (' . implode( ",", $guest['style'] ) . ');';
                $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
                $pre->execute( );
                $style = $pre->fetchall();

                foreach( $style as $product ) {
                    if( $product['category_id'] == 'C' )
                        $coat_style = $product['id'];
                    else if( $product['category_id'] == 'P' && $pant_style == null )
                        $pant_style = $product['id'];
                    else if( $product['category_id'] == 'P' && $pant_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( $product['category_id'] == 'S' && $shirt_style == null )
                        $shirt_style = $product['id'];
                    else if( $product['category_id'] == 'S' && $shirt_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( $product['category_id'] == 'J' && $jewelry_style == null )
                        $jewelry_style = $product['id'];
                    else if( $product['category_id'] == 'S' && $jewelry_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( $product['category_id'] == 'N' && $neckwear_style == null )
                        $neckwear_style = $product['id'];
                    else if( $product['category_id'] == 'N' && $neckwear_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( $product['category_id'] == 'SH' && $shoe_style == null )
                        $shoe_style = $product['id'];
                    else if( $product['category_id'] == 'SH' && $shoe_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( ( $product['category_id'] == 'V' || $product['category_id'] == 'CB' ) && $accessory_style == null )
                        $accessory_style = $product['id'];
                    else if( ( $product['category_id'] == 'V' || $product['category_id'] == 'CB' ) && $accessory_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( $product['category_id'] == 'H' && $other_style == null )
                        $other_style = $product['id'];
                    else if( $product['category_id'] == 'H' && $other_style != null ) {
                        if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                }
            }

            $sql = "INSERT INTO `tblEventParticipant` (`event_id`,
                                                        `origin_store_id`,
                                                        `employee_id`,
                                                        `customer_id`,
                                                        `event_role_id`,
                                                        `date_created`,
                                                        `coat_style`,
                                                        `pant_style`,
                                                        `shirt_style`,
                                                        `jewelry_style`,
                                                        `accessory_style`,
                                                        `neckwear_style`,
                                                        `shoe_style`,
                                                        `other_style`,
                                                        `other_2_style`,
                                                        `other_3_style`,
                                                        `webstore_product_variation`,
                                                        `measurement_id`,
                                                        `comment`,
                                                        `discount_code`) VALUES (
                                                        :event_id,
                                                        :origin_store_id,
                                                        :employee_id,
                                                        :customer_id,
                                                        :event_role_id,
                                                        NOW(),
                                                        :coat_style,
                                                        :pant_style,
                                                        :shirt_style,
                                                        :jewelry_style,
                                                        :accessory_style,
                                                        :neckwear_style,
                                                        :shoe_style,
                                                        :other_style,
                                                        :other_2_style,
                                                        :other_3_style,
                                                        :webstore_product_variation,
                                                        :measurement_id,
                                                        :comment,
                                                        :discount_code);";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':event_id'                     => $event_id,
                ':origin_store_id'              => $usr->baseStoreId,
                ':employee_id'                  => $usr->id,
                ':customer_id'                  => $customer_id,
                ':event_role_id'                => $guest['type'],
                ':coat_style'                   => $coat_style,
                ':pant_style'                   => $pant_style,
                ':shirt_style'                  => $shirt_style,
                ':jewelry_style'                => $jewelry_style,
                ':accessory_style'              => $accessory_style,
                ':neckwear_style'               => $neckwear_style,
                ':shoe_style'                   => $shoe_style,
                ':other_style'                  => $other_style,
                ':other_2_style'                => $other_2_style,
                ':other_3_style'                => $other_3_style,
                ':webstore_product_variation'   => $guest['webstore_product'],
                ':measurement_id'               => empty( $guest['rentalMeasurements'] ) ? null : $guest['rentalMeasurements']['measurementId'],
                ':comment'                      => $guest['comment'],
                ':discount_code'                => implode( ',', $guest['promo'] )
            );
            $pre->execute( $values );
        }

        // Create reservations for all the guests on the guest list
        $sql = "call spEventToReservation(:event_id)";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':event_id'   => $event_id
        );
        $pre->execute( $values );

        // Create event on Wintux
        $sql = "select name from tblEventType where id = :event_type;";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':event_type'   => $params['event_type']
        );
        $pre->execute( $values );
        $event_type = $pre->fetch()['name'];

        $event = array(
            'type'      => $event_type,
            'date'      => $params['event_date'],
            'location'  => $params['location'],
            'groom'     => array(
                'firstName'     => $params['organizers'][0]['firstName'],
                'lastName'      => $params['organizers'][0]['lastName'],
                'phone'         => $params['organizers'][0]['phone'],
                'email'         => $params['organizers'][0]['email']
            ),
            'bride'     => array(
                'firstName'     => empty( $params['organizers'][1] ) ? '' : $params['organizers'][1]['firstName'],
                'lastName'      => empty( $params['organizers'][1] ) ? '' : $params['organizers'][1]['lastName'],
                'phone'         => empty( $params['organizers'][1] ) ? '' : $params['organizers'][1]['phone'],
                'email'         => empty( $params['organizers'][1] ) ? '' : $params['organizers'][1]['email']
            ),
            'guests'    => array()
        );

        $wintux = new Wintux( 'pr', false, true );
        $wintux_event = $wintux->create_Event( $event );

        return $response->withJson(array('status' => true, 'error' => false, 'eventId' => $wintux_event), 200); //$event_id
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

/*
 * To create an event the endpoint need the following information
 * 
 * event: array(
 *  id:                                                     // ID of the vent that is going 
 *                                                             to be updated
 *  main_contact_name:      varchar(100)                    // main contact of the event        *
 *  main_contact_phone:     varchar(15)                     // main contact phone number        *
 *  main_contact_email:     varchar(100)                    // main contact email               *
 *  occasion:               varchar(50)                     // event name                       *
 *  event_type:             tinyint - options               // event type                       *
 *                              1: Wedding, 
 *                              2: Prom, 
 *                              3: Quinceañera, 
 *                              4: Gala, 
 *                              5: Other
 *  location:               varchar(75)                     // even location                    *
 *  event_date:             date                            // event date                       *
 *  discount_code:          varchar(45)                     // discount code
 *  employee_id:            int,                            // id of the employee that helped   *
 *  referral_source:        smallint - options:             // how the customer hear about us
 *                              1: Billboard, 
 *                              2: Bridal Show, 
 *                              3: Competitors Coupon, 
 *                              4: Direct Mail Coupon, 
 *                              5: Email Blast,
 *                              6: Internet / Website,
 *                              7: Newspaper,
 *                              8: Referred By Friend,
 *                              9: Radio,
 *                              10: Television,
 *                              11: VIP Card,
 *                              12: Walk-in,
 *                              13: Telemarketing,
 *                              14: Facebook,
 *                              15: Instagram,
 *                              16: Wedding Coordinator,
 *  guests:                 array(                          // guests information               *
 *      customer_id:            int - null if is new        // id of already created customer
 *      type:                   smallint - options:         // type of customer                 
 *                                  1: Person, 
 *                                  2: Company,
 *      first_name:             varchar(50)                 // first name
 *      last_name:              varhcar(50)                 // last name
 *      full_name:              varchar(101)                // full name - only required when 
 *                                                             first name and last name are not 
 *                                                             provided
 *      phone:                  varchar(15)                 // customer phone number
 *      email:                  varchar(100)                // customer email address
 *      gender:                 varchar(1) - options:       // customer gender
 *                                  m: Male, 
 *                                  f: Female, 
 *                                  n: neutral
 *      address_line_1:         varchar(40)                 // customer address line 1
 *      address_line_2:         varchar(40)                 // customer address line 2
 *      address_city:           varchar(25)                 // customer address city
 *      address_state:          varchar(2)                  // customer address state
 *      address_zipcode:        varchar(10)                 // customer address zip code
 *      event_role              smallint - options:         // guest role on the event
 *                                  1: Groom
 *                                  2: Gala
 *                                  3: Wedding Guest
 *                                  4: Aniversary
 *                                  5: Ship
 *                                  6: Birthday
 *                                  7: Quinceañero
 *                                  8: Comitiva
 *                                  9: Other
 *                                  10: Student
 *                                  11: Best Man
 *                                  12: Family Member
 *                                  13: Groomsman
 *                                  14: Usher
 *      coat_style:             varchar(10)                 // coat style sku / upc
 *      pant_style:             varchar(10)                 // pant style sku / upc
 *      shirt_style:            varchar(10)                 // shirt style sku / upc
 *      jewelry_style:          varchar(10)                 // jewelry style sku / upc
 *      accessory_style:        varchar(10)                 // accessory style sku / upc
 *      neckwear_style:         varchar(10)                 // neckwear style sku / upc
 *      shoe_style:             varchar(10)                 // shoe style sku / upc
 *      other_style:            varchar(10)                 // other style sku / upc
 *      other_2_style:          varchar(10)                 // other 2 style sku / upc
 *      other_3_style:          varchar(10)                 // other 3 style sku / upc
 *      webstore_product:       varchar(20)                 // website product id | product variation
 *  )
 * )
 */
$app->put('/event', function ($request, $response) use ($lrPluginUrl) {
    try {
        // Get all the request values
        $params = $request->getParsedBody()['event'];

        $token = (new Parser())->parse((string) $request->getAttribute('token'));         
        $usr = $token->getClaim('usr');

        $con = $this->apidb;

        // Create event
        $sql = "update `tblEvent` set `coordinator1_customer_id` = :coordinator1_customer_id,
                                      `coordinator1_comment` = :coordinator1_comment,
                                      `coordinator2_customer_id` = :coordinator2_customer_id,
                                      `coordinator2_comment` = :coordinator2_comment,
                                      `occasion` = :occasion,
                                      `location` = :location,
                                      `event_date` = :event_date,
                                      `discount_code` = :discount_code,
                                      `referral_source_id` = :referral_source_id,
                                      `comment` = :comment,
                                      `default_style` = :default_style,
                                      `discount_code` = :discount_code
                where id = :event_id";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':coordinator1_customer_id'     => $params['organizers'][0]['customerId'],
            ':coordinator1_comment'         => $params['organizers'][0]['comment'],
            ':coordinator2_customer_id'     => empty( $params['organizers'][1]['customerId'] ) ? null : $params['organizers'][1]['customerId'],
            ':coordinator2_comment'         => empty( $params['organizers'][1]['comment'] ) ? null : $params['organizers'][1]['comment'],
            ':occasion'                     => $params['occasion'],
            ':location'                     => $params['location'],
            ':event_date'                   => $params['event_date'],
            ':discount_code'                => $params['discount_code'],
            ':referral_source_id'           => $params['referral_source'],
            ':comment'                      => $params['comment'],
            ':default_style'                => $params['default_style'],
            ':discount_code'                => $params['promo'],
            ':event_id'                     => $params['id']
        );
        $pre->execute( $values );

        // Remove previously event participants
        $sql = "delete from `tblEventParticipant` where event_id = :event_id";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':event_id' => $params['id'],
        );
        $pre->execute( $values );

        // Create event participants
        foreach( $params['guests'] as $guest ) {
            // First check if their is a customer with the given phone number or email
            $customer_id = $guest['id'];
            if ( empty( $customer_id ) ) {
                $sql = "select customerId from vwCustomer where phone = :phone or email = :email";
                $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
                $values = array(
                    ':phone' => $guest['phone'],
                    ':email' => $guest['email']
                );
                $pre->execute( $values );
                $customer_id = $pre->fetch()['customerId'];
            }

            // Create customer if it doesn't exist
            if ( empty( $customer_id ) ) {

                $customer_info = (object)array(
                    'customerTypeId'        => 1,
                    'customerName'          => $guest['first_name'],
                    'customerLastName'      => $guest['last_name'],
                    'customerFullName'      => $guest['full_name'],
                    'customerPhone'         => $guest['phone'],
                    'customerEmail'         => $guest['email'],
                    'customerGender'        => $guest['gender'],
                    'customerAddressLine1'  => $guest['address_line_1'],
                    'customerAddressLine2'  => $guest['address_line_2'],
                    'customerCity'          => $guest['address_city'],
                    'customerState'         => $guest['address_state'],
                    'customerZipcode'       => $guest['address_zipcode']
                );
                
                $customer_id = createCustomer( $customer_info, $con );
            }

            // Get guest style information
            $coat_style = null;
            $pant_style = null;
            $shirt_style = null;
            $jewelry_style = null;
            $accessory_style = null;
            $neckwear_style = null;
            $shoe_style = null;
            $other_style = null;
            $other_2_style = null;
            $other_3_style = null;
            if( !empty( $guest['style'] ) ) {
                $sql = 'select id, style, category_id
                        from tblProduct
                        where id in (' . implode( ",", $guest['style'] ) . ');';
                $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
                $pre->execute( );
                $style = $pre->fetchall();

                foreach( $style as $product ) {
                    if( $product['category_id'] == 'C' )
                        $coat_style = $product['id'];
                    else if( $product['category_id'] == 'P' && $pant_style == null )
                        $pant_style = $product['id'];
                    else if( $product['category_id'] == 'P' && $pant_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( $product['category_id'] == 'S' && $shirt_style == null )
                        $shirt_style = $product['id'];
                    else if( $product['category_id'] == 'S' && $shirt_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( $product['category_id'] == 'J' && $jewelry_style == null )
                        $jewelry_style = $product['id'];
                    else if( $product['category_id'] == 'S' && $jewelry_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( $product['category_id'] == 'N' && $neckwear_style == null )
                        $neckwear_style = $product['id'];
                    else if( $product['category_id'] == 'N' && $neckwear_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( $product['category_id'] == 'SH' && $shoe_style == null )
                        $shoe_style = $product['id'];
                    else if( $product['category_id'] == 'SH' && $shoe_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( ( $product['category_id'] == 'V' || $product['category_id'] == 'CB' ) && $accessory_style == null )
                        $accessory_style = $product['id'];
                    else if( ( $product['category_id'] == 'V' || $product['category_id'] == 'CB' ) && $accessory_style != null ) {
                        if( $other_style == null )
                            $other_style = $product['id'];
                        else if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                    else if( $product['category_id'] == 'H' && $other_style == null )
                        $other_style = $product['id'];
                    else if( $product['category_id'] == 'H' && $other_style != null ) {
                        if( $other_2_style == null )
                            $other_2_style = $product['id'];
                        else if( $other_3_style == null )
                            $other_3_style = $product['id'];
                    }
                }
            }

            $sql = "INSERT INTO `tblEventParticipant` (`event_id`,
                                                        `origin_store_id`,
                                                        `employee_id`,
                                                        `customer_id`,
                                                        `event_role_id`,
                                                        `date_created`,
                                                        `coat_style`,
                                                        `pant_style`,
                                                        `shirt_style`,
                                                        `jewelry_style`,
                                                        `accessory_style`,
                                                        `neckwear_style`,
                                                        `shoe_style`,
                                                        `other_style`,
                                                        `other_2_style`,
                                                        `other_3_style`,
                                                        `webstore_product_variation`,
                                                        `measurement_id`,
                                                        `comment`,
                                                        `discount_code`) VALUES (
                                                        :event_id,
                                                        :origin_store_id,
                                                        :employee_id,
                                                        :customer_id,
                                                        :event_role_id,
                                                        NOW(),
                                                        :coat_style,
                                                        :pant_style,
                                                        :shirt_style,
                                                        :jewelry_style,
                                                        :accessory_style,
                                                        :neckwear_style,
                                                        :shoe_style,
                                                        :other_style,
                                                        :other_2_style,
                                                        :other_3_style,
                                                        :webstore_product_variation,
                                                        :measurement_id,
                                                        :comment,
                                                        :discount_code);";
            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                ':event_id'                     => $params['id'],
                ':origin_store_id'              => $usr->baseStoreId,
                ':employee_id'                  => $usr->id, 
                ':customer_id'                  => $customer_id,
                ':event_role_id'                => $guest['type'],
                ':coat_style'                   => $coat_style,
                ':pant_style'                   => $pant_style,
                ':shirt_style'                  => $shirt_style,
                ':jewelry_style'                => $jewelry_style,
                ':accessory_style'              => $accessory_style,
                ':neckwear_style'               => $neckwear_style,
                ':shoe_style'                   => $shoe_style,
                ':other_style'                  => $other_style,
                ':other_2_style'                => $other_2_style,
                ':other_3_style'                => $other_3_style,
                ':webstore_product_variation'   => $guest['webstore_product'],
                ':measurement_id'               => empty( $guest['rentalMeasurements'] ) ? null : $guest['rentalMeasurements']['measurementId'],
                ':comment'                      => $guest['comment'],
                ':discount_code'                => implode( ',', $guest['promo'] )
            );
            $pre->execute( $values );
        }

        return $response->withJson(array('status' => true, 'error' => false, 'eventId' => $event_id), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()), 422);
    }
});

/*************************************
************* EMPLOYEES **************
*************************************/

$app->get('/employee/validate/{security_code}', function ($request, $response) use ($lrPluginUrl) {
    try {
        $con = $this->apidb;
        $sql = "select id, first_name, last_name, full_name, address_line_1 from tblEmployee where security_code = :security_code and active = 1";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':security_code' => $request->getAttribute('security_code')
        );
        $pre->execute( $values );
        $result = $pre->fetch();

        if(!$result) 
            return $response->withJson(array('status' => false, 'error' => true, 'message' => 'Invalid security code'), 200);

        $employee = array(
            'id'        => $result['id'],
            'firstName' => $result['first_name'],
            'lastName'  => $result['last_name'],
            'fullName'  => $result['full_name']
        );

        return $response->withJson(array('status' => true, 'error' => false,'data'=> $employee), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

/*************************************
************** HELPERS ***************
*************************************/

$app->get('/pickup-locations', function ($request, $response) use ($lrPluginUrl) {
    try {
        $con = $this->apidb;
        $sql = "select id, name, charge from tblPickupLocation where active = 1";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute( );
        $result = $pre->fetchall();

        if(!$result) 
            return $response->withJson(array('status' => false, 'error' => true, 'message' => 'Pickup locations not configured.'), 200);

        return $response->withJson(array('status' => true, 'error' => false,'data'=> $result), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

$app->get('/promo-codes', function ($request, $response) use ($lrPluginUrl) {
    try {
        $con = $this->apidb;
        $sql = "select code, description, use_with_others, amount, min_amount, max_amount, min_items, max_items
                from tblPromoCode
                where active = 1";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute();
        $result = $pre->fetchall();

        if(!$result) 
            return $response->withJson(array('status' => false, 'error' => true, 'message' => 'Promo codes not configured.'), 200);

        return $response->withJson(array('status' => true, 'error' => false,'data'=> $result), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

$app->get('/products', function ($request, $response) use ($lrPluginUrl) {
    try {
        $con = $this->apidb;
        $sql = "select productId id, style foreignId, categoryId, category, description name, price, salePrice
                from vwProducts
                where active = 1
                order by categoryId";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute();
        $products = $pre->fetchall();

        if(!$products) 
            return $response->withJson(array('status' => false, 'error' => true, 'message' => 'Products not configured.'), 200);

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $products ), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

$app->get('/extra-items', function ($request, $response) use ($lrPluginUrl) {
    try {
        $con = $this->apidb;
        $sql = "SELECT id, name, type_code categoryId, amount
                FROM leonard5_lfa_main.tblExtraItem
                where active = 1;";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute();
        $extra_items = $pre->fetchall();

        if(!$extra_items) 
            return $response->withJson(array('status' => false, 'error' => true, 'message' => 'Exta items not configured.'), 200);

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $extra_items ), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

$app->get('/referral-sources', function ($request, $response) use ($lrPluginUrl) {
    try {
        $con = $this->apidb;
        $sql = "SELECT id, name
                FROM leonard5_lfa_main.tblEventReferralSource;";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute();
        $sources = $pre->fetchall();

        if(!$sources) 
            return $response->withJson(array('status' => false, 'error' => true, 'message' => 'Referral sources are not configured.'), 200);

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $sources ), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

$app->get('/check-availability/{sku}/{type}', function ($request, $response) use ($lrPluginUrl) {
    try {
        $sku = $request->getAttribute('sku');
        $type = $request->getAttribute('type');

        $size = explode( '||', $sku )[1];
        $event_date = explode( '||', $sku )[2];
        $sku = explode( '||', $sku )[0];

        if( $type == 'P' ) {
            if( in_array( $size, array( 27, 28, 29 ) ) ) $size = '28R'; 
            else if( in_array( $size, array( 30, 31, 32 ) ) ) $size = '31R'; 
            else if( in_array( $size, array( 33, 34, 35 ) ) ) $size = '34R'; 
            else if( in_array( $size, array( 36, 37, 38 ) ) ) $size = '37R'; 
            else if( in_array( $size, array( 39, 40, 41 ) ) ) $size = '40R'; 
            else if( in_array( $size, array( 42, 43, 44 ) ) ) $size = '43R'; 
            else if( in_array( $size, array( 45, 46, 47 ) ) ) $size = '46R'; 
            else if( in_array( $size, array( 48, 49, 50 ) ) ) $size = '49R'; 
            else if( in_array( $size, array( 51, 52, 53 ) ) ) $size = '52R'; 
            else if( in_array( $size, array( 54, 55, 56 ) ) ) $size = '55R'; 
            else if( in_array( $size, array( 57, 58, 59 ) ) ) $size = '58R'; 
            else if( in_array( $size, array( 60, 61, 62 ) ) ) $size = '61R'; 
            else if( in_array( $size, array( 63, 64, 65 ) ) ) $size = '64R'; 
            else if( in_array( $size, array( 66, 67, 68 ) ) ) $size = '67R'; 
        }
        
        $wintux = new Wintux('pr', false, true);
        $r = $wintux->is_item_available($sku, $size, $event_date);
		
        if ( $r['error'] ) {
            return $response->withJson(array('status' => false, 'error' => true, 'message' => $r['message'] ), 200);  
        }

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $r['content'] ), 200);       
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()),422);
    }
})->add($mw_authorize);

$app->get('/store/list', function ($request, $response) use ($lrPluginUrl) {
    try {
        $con = $this->apidb;
        $sql = "SELECT id, name, authKey, typeId, type 
                FROM vwStore
                where active = 1 and typeId = 5
                order by name asc;";
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute();
        $stores = $pre->fetchall();

        if(!$stores) 
            return $response->withJson(array('status' => false, 'error' => true, 'message' => 'Stores are not configured.'), 200);

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $stores ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()),422);
    }
});

$app->post('/store/open', function ($request, $response) use ($lrPluginUrl) {
    try {
        $params = $request->getParsedBody();

        $code = $params['code'];

        $con = $this->apidb;
        $sql = "select store_id 
            from tblStoreCodes 
            where code = :code and used + 1 <= max_use and now() <= date_expire";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':code' => $code
        );
        $pre->execute($values);
        $store_id = $pre->fetch()['store_id'];

        if(!$store_id) 
            return $response->withJson(array('status' => false, 'error' => true, 'message' => 'Invalid code.'), 200);     

        // Closeing time: 11 hours after the open time (9 working hours + 2 hours to fix the timezone issue [PR time])
        $sql = "select id, name, authKey, typeId, type, phone, wintuxStoreId, DATE_ADD(now(), INTERVAL 11 HOUR) closingTime
                FROM vwStore
                where active = 1 and id = :store_id;";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':store_id' => $store_id
        );
        $pre->execute($values);
        $store = $pre->fetch();

        if(!$store) 
            return $response->withJson(array('status' => false, 'error' => true, 'message' => 'Store doesn\'t exist or is not active.'), 200);     

        $sql = "update tblStoreCodes set used = used + 1
                where code = :code";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':code' => $code
        );
        $pre->execute($values);

        // poner código para identificar la tienda como abierta

        // poner código de LOG

        return $response->withJson(array('status' => true, 'error' => false, 'data' => $store, 'message' => 'Store open' ), 200);
    }
    catch(\Exception $ex) {
        return $response->withJson(array('error' => $ex->getMessage()),422);
    }
});

$app->post('/employee/signin', function($request, $response) {
    try {
        // Get all the request values
        // $params = getOrderParams( $request );
        $params = $request->getParsedBody()['form'];

        $code = $params['code'];

        if ( strlen($code) == 0 )
            return $response->withJson( array('status' => false, 'error' => false, 'message' => 'Security code is required.' ), 200 );

        $con = $this->apidb;
        $sql = "select * 
                from vwEmployee 
                where securityCode = :code";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
                    ':code' => $code
                );
        $pre->execute($values);
        $userVals = $pre->fetch();

        // If no account found
        if ( empty($userVals) )
            return $response->withJson( array('status' => false, 'error' => false, 'message' => 'Employee not found.' ), 200 );

        // Create object of the user to include on the jwt
        $user = (object)array(
            'id'                    => $userVals['employeeId'],
            'firstName'             => $userVals['firstName'],
            'lastName'              => $userVals['lastName'],
            'fullName'              => $userVals['fullName'],
            'clerkNum'              => $userVals['wintuxClerkNum'],
            'workPhone'             => $userVals['workPhone'],
            'mobilePhone'           => $userVals['mobilePhone'],
            'email'                 => $userVals['email'],
            'departmentId'          => $userVals['departmentId'],
            'department'            => $userVals['department'],
            'positionId'            => $userVals['positionId'],
            'position'              => $userVals['position'],
            'eligibleForCommission' => $userVals['eligibleForCommission'] == '1' ? true : false,
            'baseStoreId'           => $userVals['baseStoreId'],
            'baseStore'             => $userVals['baseStore'],
            'active'                => $userVals['active'] == '1' ? true : false,
            'dateCreated'           => $userVals['dateCreated'],
            'lastLogin'             => $userVals['lastlogin'],
            'rmhPassword'           => $userVals['rmhPassword']
            // 'privileges'            => array( 
            //     'allowViewHouse'                    => $userVals['allowViewHouse'] == 1 ? true : false,
            //     'allowResidentMessages'             => $userVals['allowResidentMessages'] == 1 ? true : false,
            //     'allowListing'                      => $userVals['allowListing'] == 1 ? true : false,
            //     'allowOccupation'                   => $userVals['allowOccupation'] == 1 ? true : false,
            //     'allowCommunityMessageNotification' => $userVals['allowCommunityMessageNotification'] == 1 ? true : false,
            //     'allowNewEventNotification'         => $userVals['allowNewEventNotification'] == 1 ? true : false,
            //     'allowMessagesNotification'         => $userVals['allowMessagesNotification'] == 1 ? true : false,
            //     'allowPaymentsNotification'         => $userVals['allowPaymentsNotification'] == 1 ? true : false,
            // ),
        );

        // Update user last login
        $sql = "update tblEmployee set last_login = NOW() where security_code = :code";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
                    ':code' => $code
                );
        $pre->execute($values);

        // Generate JWT
        $signer = new Sha256();
        $secret_key = 'LF@!-' . $userVals['employeeId'] . '-' . time();
        $token = (new Builder())->setIssuer('https://leonardofifthavenue.com') // Configures the issuer (iss claim)
                        ->setAudience('lfa-rental-kiosk') // Configures the audience (aud claim)
                        ->setId('4f1g23a12aa', true) // Configures the id (jti claim), replicating as a header item
                        ->setIssuedAt(time()) // Configures the time that the token was issue (iat claim)
                        ->setNotBefore(time()) // Configures the time that the token can be used (nbf claim)
                        ->setExpiration(time() + 900 ) // Configures the expiration time of the token (exp claim) (15 mins)
                        ->set('usr', $user) // Configures a new claim, called "username"
                        ->sign($signer, $secret_key) // creates a signature using "testing" as key
                        ->getToken(); // Retrieves the generated token

        // Return success response code
        return $response->withJson(
            array('status'  => true,
                  'error'   => false,
                  'token'   => $token->__toString() ), 200 
        );

    }
    catch( \Exception $ex ) {
        return $response->withJson(array('error' => $ex->getMessage()), 422);
    }
});

/*************************************
**************** CORS ****************
*************************************/

// Catch-all route to serve a 404 Not Found page if none of the routes match
// NOTE: make sure this route is defined last
$app->map(['GET', 'POST', 'PUT', 'DELETE', 'PATCH'], '/{routes:.+}', function($req, $res) {
    $handler = $this->notFoundHandler; // handle using the default Slim page not found handler
    return $handler($req, $res);
});


/*************************************
************* FUNCTIONS **************
*************************************/

function lfa_create_order( $params, $client, $update_customer, $update_customer_measurements, $con ) {

    // Validate request parameters
    // $errors = validateOrderParameters( $params );

    // // If there are errors return the error messages
    // if( count( $errors ) > 0 ) {
    //     return $response->withJson(array('status' => 'error', 'errors' => $errors), 200);
    // }

    // Create customer if is new
    $customer_id = 0;
    $sql = "select id customer_id from tblCustomer where email = :email or phone = :phone";
    $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
    $values = array(
        ':email' => $params['customer']['email'],
        ':phone' => $params['customer']['customerPhone']
    );
    $pre->execute( $values );
    $customer_id = $pre->fetch()['customer_id'];

    // if customer doesn't exist, create new customer
    if( empty( $customer_id ) ) {
        // customer information
        $customer = (object)array(
            'customerTypeId'        => 1,
            'customerName'          => $params['customer']['first_name'],
            'customerLastName'      => $params['customer']['last_name'],
            'customerFullName'      => '',
            'customerPhone'         => $params['customer']['phone'],
            'customerEmail'         => $params['customer']['email'],
            'customerGender'        => $params['customer']['gender'],
            'customerAddressLine1'  => $params['customer']['address_line_1'],
            'customerAddressLine2'  => $params['customer']['address_line_2'],
            'customerCity'          => $params['customer']['address_city'],
            'customerState'         => $params['customer']['address_state'],
            'customerZipcode'       => $params['customer']['address_zipcode']
        );
        $customer_id = createCustomer ( $customer, $con );
        
        // customer measurements
        saveCustomerMeasurements( (object)$params['customer']['measurements'], $customer_id, $con );
    }
    else {
        // update customer information 
        if ( $update_customer ) {
            $customer = (object)array(
                'customerTypeId'        => 1,
                'customerName'          => $params['customer']['first_name'],
                'customerLastName'      => $params['customer']['last_name'],
                'customerFullName'      => '',
                'customerPhone'         => $params['customer']['phone'],
                'customerEmail'         => $params['customer']['email'],
                'customerGender'        => $params['customer']['gender'],
                'customerAddressLine1'  => $params['customer']['address_line_1'],
                'customerAddressLine2'  => $params['customer']['address_line_2'],
                'customerCity'          => $params['customer']['address_city'],
                'customerState'         => $params['customer']['address_state'],
                'customerZipcode'       => $params['customer']['address_zipcode']
            );
            updateCustomer( $customer_id, $customer, $con );   
        }

        // update customer measurements
        if( $update_customer_measurements ) {
            saveCustomerMeasurements( $params['customer']['measurements'], $customer_id, $con );
        }
    }

    // transform order status
    $order_status_id = 0;
    $sql = "select id from tblOrderStatus where foreign_id = :foreign_id";
    $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
    $values = array(
        ':foreign_id' => $params['status']
    );
    $pre->execute($values);
    $order_status_id = $pre->fetch()['id'];

    // Create order
    $sql = "INSERT INTO `tblOrder` (`foreign_id`, 
                                    `order_date`,
                                    `customer_id`, 
                                    `foreign_customer_id`, 
                                    `subtotal`,
                                    `other_charges_discount`, 
                                    `tax`, 
                                    `shipping`, 
                                    `total`,
                                    `status_id`, 
                                    `pickup_date`, 
                                    `referral_code`, 
                                    `store_id`, 
                                    `employee_id`, 
                                    `use_date`, 
                                    `want_date`, 
                                    `return_date`) VALUES (
                                    :foreign_id,
                                    :order_date,
                                    :customer_id,
                                    :foreign_customer_id,
                                    :subtotal,
                                    :other_charges_discount,
                                    :tax,
                                    :shipping,
                                    :total,
                                    :status_id,
                                    :pickup_date,
                                    :referral_code,
                                    :store_id,
                                    :employee_id,
                                    :use_date,
                                    :want_date,
                                    :return_date)";
    $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
    $values = array(
                    ':foreign_id'               => $params['order_id'],
                    ':order_date'               => $params['order_date'],
                    ':customer_id'              => $customer_id,
                    ':foreign_customer_id'      => $params['customer']['foreign_id'],
                    ':subtotal'                 => $params['subtotal'],
                    ':other_charges_discount'   => $params['other_charges_discount'],
                    ':tax'                      => $params['tax'],
                    ':shipping'                 => $params['shipping'],
                    ':total'                    => $params['total'],
                    ':status_id'                => $order_status_id,
                    ':pickup_date'              => $params['pickup_date'],
                    ':referral_code'            => $params['referral_code'],
                    ':store_id'                 => $client['id'],
                    ':employee_id'              => $params['employee_id'],
                    ':use_date'                 => $params['use_date'],
                    ':want_date'                => $params['want_date'],
                    ':return_date'              => $params['return_date']
                );
    $pre->execute($values);

    // Get order id
    $sql = "SELECT LAST_INSERT_ID() order_id;";
    $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
    $pre->execute();
    $order_id = $pre->fetch()['order_id'];

    // Save order details
    foreach( $params['details'] as $item ) {
        $sql = "INSERT INTO `tblOrderDetail` (`order_id`,
                                        `order_type_id`,
                                        `product_id`,
                                        `product_name`, 
                                        `qty`,
                                        `price`,
                                        `total`,
                                        `image`,
                                        `is_couture`,
                                        `event_participant_id`) VALUES (
                                        :order_id,
                                        :order_type_id,
                                        :product_id,
                                        :product_name,
                                        :qty,
                                        :price,
                                        :total,
                                        :image,
                                        :is_couture,
                                        :event_participant_id)";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
                        ':order_id'             => $order_id,
                        ':order_type_id'        => $item['order_type'],
                        ':product_id'           => $item['product_id'],
                        ':product_name'         => $item['product_name'],
                        ':qty'                  => $item['qty'],
                        ':price'                => $item['price'],
                        ':total'                => $item['total'],
                        ':image'                => $item['image'],
                        ':is_couture'           => $item['is_couture'],
                        ':event_participant_id' => $item['event_participant_id']
                    );
        $pre->execute( $values );

        $sql = "SELECT LAST_INSERT_ID() item_id";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute( );
        $item_id = $pre->fetch()['item_id'];

        // Save item modifications
        foreach( $item['modifications'] as $mod ) {

            $sql = "INSERT INTO `tblOrderDetailCustomization` (`order_id`,
                                        `item_id`, 
                                        `customization_type`, 
                                        `category_id`,
                                        `category_name`,
                                        `option_id`,
                                        `option_name`,
                                        `option_image`,
                                        `couture_description`) VALUES (
                                        :order_id,
                                        :item_id,
                                        :customization_type,
                                        :category_id,
                                        :category_name,
                                        :option_id,
                                        :option_name,
                                        :option_image,
                                        :couture_description)";
            $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
            $values = array(
                            ':order_id' => $order_id,
                            ':item_id' => $item_id,
                            ':customization_type' => $mod['modification_type'],
                            ':category_id' => $mod['categor_id'],
                            ':category_name' => $mod['category_name'],
                            ':option_id' => $mod['option_id'],
                            ':option_name' => $mod['option_name'],
                            ':option_image' => $mod['option_image'],
                            ':couture_description' => $mod['couture_description']
                        );
            $pre->execute( $values );
        }
    }

    // Save payment
    foreach( $params['payments'] as $payment ) {
        $sql = "INSERT INTO `tblOrderPayment` (`order_id`, 
                                        `payment_date`,
                                        `payment_amount`, 
                                        `memo`,
                                        `payment_method`,
                                        `payment_confirmation_number`) VALUES (
                                        :order_id,
                                        :payment_date,
                                        :payment_amount,
                                        :memo,
                                        :payment_method,
                                        :payment_confirmation_number)";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':order_id'                     => $order_id,
            ':payment_date'                 => $payment['date'],
            ':payment_amount'               => floatval( str_replace( ',', '', $payment['amount'] ) ),
            ':memo'                         => $payment['description'],
            ':payment_method'               => $payment['method'],
            ':payment_confirmation_number'  => $payment['confirmation_number']
        );
        $pre->execute($values);
    }

    // Save order special charges
    foreach($params['charges_discounts'] as $cd) {
        $sql = "INSERT INTO `tblOrderOtherCharges` (`order_id`,
                                                    `other_charges_type`,
                                                    `amount`,
                                                    `memo`,
                                                    `code`)
                                                    VALUES
                                                    (:order_id,
                                                     :other_charges_type,
                                                     :amount,
                                                     :memo,
                                                     :code);";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
                        ':order_id'             => $order_id,
                        ':other_charges_type'   => $cd['other_charges_type'],
                        ':amount'               => floatval( $cd['amount'] ),
                        ':memo'                 => $cd['description'],
                        ':code'                 => $cd['code']
                    );
        $pre->execute($values);
    }

    // // Register order creation in order history
    // $sql = "insert into `tblOrderHistory` values (:order_id, now(), 'system', 'Order created', 'creation', null)";
    // $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
    // $values = array(
    //                 ':order_id' => $order_id,
    //             );
    // $pre->execute($values);

    // // Register payment in order history
    // $sql = "insert into `tblOrderHistory` values (:order_id, now(), 'system', :message, 'payment', null)";
    // $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
    // $values = array(
    //                 ':order_id' => $order_id,
    //                 ':message' => 'A $' . $params->paymentAmount . ' USD payment was processed ' . ($params->paymentMethod == 'Stripe' ? 'on Stripe' : 'in store')
    //             );
    // $pre->execute($values);

    // // log action
    // $log = new LogQueries();
    // $sql = $log->OrderCreation($order_id, $params->customerName . ' ' . $params->customerLastName, $request->getAttribute('ip_address'), '', '');
    // $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
    // $pre->execute( );

    return true;
}

function createCustomer ( $params, $con ) {
    $sql = "insert into `tblCustomer` (`customer_type_id`,
                                        `first_name`,
                                        `last_name`,
                                        `full_name`,
                                        `birthdate`,
                                        `phone`,
                                        `email`,
                                        `gender`,
                                        `address_line_1`,
                                        `address_line_2`,
                                        `address_city`,
                                        `address_state`,
                                        `address_zipcode`,
                                        `date_created`) values (
                                        :customer_type_id,
                                        :first_name,
                                        :last_name,
                                        :full_name,
                                        :birthdate,
                                        :phone,
                                        :email,
                                        :gender,
                                        :address_line_1,
                                        :address_line_2,
                                        :address_city,
                                        :address_state,
                                        :address_zipcode,
                                        now())";
    $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

    $full_name = $params->customerName . ' ' . $params->customerLastName;
    if ( empty( $params->customerName ) && empty( $params->customerLastName ) )
        $full_name = $params->customerFullName;

    $values = array(
        ':customer_type_id' => empty( $params->customerTypeId ) ? 1 : $params->customerTypeId,
        ':first_name'       => $params->customerName,
        ':last_name'        => $params->customerLastName,
        ':full_name'        => $full_name,
        ':birthdate'        => $params->customerBirthdate,
        ':phone'            => $params->customerPhone,
        ':email'            => $params->customerEmail,
        ':gender'           => $params->customerGender,
        ':address_line_1'   => $params->customerAddressLine1,
        ':address_line_2'   => $params->customerAddressLine2,
        ':address_city'     => $params->customerCity,
        ':address_state'    => $params->customerState,
        ':address_zipcode'  => $params->customerZipcode
    );
    $pre->execute( $values );

    $sql = "SELECT LAST_INSERT_ID() customer_id;";
    $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
    $pre->execute();
    $customer_id = $pre->fetch()['customer_id'];

    return $customer_id;
}

function updateCustomer( $identifier, $params, $con ) {
    $sql = ''; 
    $values = array();
    if( !empty( $params->customerTypeId ) ) {
        $sql .= "`customer_type_id` = :customer_type_id, ";
        $values[':customer_type_id'] = $params->customerTypeId;
    }
    if( !empty( $params->customerName ) ) {
        $sql .= "`first_name` = :first_name, ";
        $values[':first_name'] = $params->customerName;
    }
    if( !empty( $params->customerLastName ) ) {
        $sql .= "`last_name` = :last_name, ";
        $values[':last_name'] = $params->customerLastName;
    }
    if( !empty( $params->customerFullName ) ) {
        $sql .= "`full_name` = :full_name, ";
        $values[':full_name'] = $params->customerFullName;
    }
    else 
        $sql .= "`full_name` = concat(first_name, ' ', last_name), ";
    if( !empty( $params->customerBirthdate ) ) {
        $sql .= "`birthdate` = :birthdate, ";
        $values[':birthdate'] = $params->customerBirthdate;
    }
    if( !empty( $params->customerPhone ) ) {
        $sql .= "`phone` = :phone, ";
        $values[':phone'] = $params->customerPhone;
    }
    if( !empty( $params->customerEmail ) ) {
        $sql .= "`email` = :email, ";
        $values[':email'] = $params->customerEmail;
    }
    if( !empty( $params->customerGender ) ) {
        $sql .= "`gender` = :gender, ";
        $values[':gender'] = $params->customerGender;
    }
    if( !empty( $params->customerAddressLine1 ) ) {
        $sql .= "`address_line_1` = :address_line_1, ";
        $values[':address_line_1'] = $params->customerAddressLine1;
    }
    if( !empty( $params->customerAddressLine2 ) ) {
        $sql .= "`address_line_2` = :address_line_2, ";
        $values[':address_line_2'] = $params->customerAddressLine2;
    }
    if( !empty( $params->customerCity ) ) {
        $sql .= "`address_city` = :address_city, ";
        $values[':address_city'] = $params->customerCity;
    }
    if( !empty( $params->customerState ) ) {
        $sql .= "`address_state` = :address_state, ";
        $values[':address_state'] = $params->customerState;
    }
    if( !empty( $params->customerZipcode ) ) {
        $sql .= "`address_zipcode` = :address_zipcode, ";
        $values[':address_zipcode'] = $params->customerZipcode;
    }

    if( empty( $sql ) ) 
        return false;

    $sql = "update `tblCustomer` set " . rtrim($sql, ', ') . ' where (id = :identifier or phone = :identifier) ORDER BY id LIMIT 1';
    $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
    $values[':identifier'] = $identifier['identifier'];
    $pre->execute( $values );

    return true;
}

function saveCustomerMeasurements( $params, $customer_id, $con ) {
    $sql = "insert into `tblCustomerMeasurements` (`customer_id`,
                                                    `height_feet`,
                                                    `height_inch`,
                                                    `weight`,
                                                    `neck`,
                                                    `shoulder`,
                                                    `sleeve_length_left`,
                                                    `sleeve_length_right`,
                                                    `bicep`,
                                                    `waist`,
                                                    `wrist`,
                                                    `chest`,
                                                    `stomach`,
                                                    `jacket_length_front`,
                                                    `jacket_length_back`,
                                                    `hips`,
                                                    `thigh`,
                                                    `crotch`,
                                                    `knee`,
                                                    `leg_length`,
                                                    `vest_length_front`,
                                                    `vest_length_back`,
                                                    `cuff`,
                                                    `watch_hand`,
                                                    `calf`,
                                                    `coat_sleeve_left`,
                                                    `coat_sleeve_right`,
                                                    `date_created`) values (
                                                    :customer_id,
                                                    :height_feet,
                                                    :height_inch,
                                                    :weight,
                                                    :neck,
                                                    :shoulder,
                                                    :sleeve_length_left,
                                                    :sleeve_length_right,
                                                    :bicep,
                                                    :waist,
                                                    :wrist,
                                                    :chest,
                                                    :stomach,
                                                    :jacket_length_front,
                                                    :jacket_length_back,
                                                    :hips,
                                                    :thigh,
                                                    :crotch,
                                                    :knee,
                                                    :leg_length,
                                                    :vest_length_front,
                                                    :vest_length_back,
                                                    :cuff,
                                                    :watch_hand,
                                                    :calf,
                                                    :coat_sleeve_left,
                                                    :coat_sleeve_right,
                                                    now())";
    $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
    $values = array(
        ':customer_id'                  => $customer_id,
        ':height_feet'                  => $params->height_feet,
        ':height_inch'                  => $params->height_inch,
        ':weight'                       => $params->weight,
        ':neck'                         => $params->neck,
        ':shoulder'                     => $params->shoulder,
        ':sleeve_length_left'           => $params->sleeve_length_left,
        ':sleeve_length_right'          => $params->sleeve_length_right,
        ':bicep'                        => $params->bicep,
        ':waist'                        => $params->waist,
        ':wrist'                        => $params->wrist,
        ':chest'                        => $params->chest,
        ':stomach'                      => $params->stomach,
        ':jacket_length_front'          => $params->jacket_length_front,
        ':jacket_length_back'           => $params->jacket_length_back,
        ':hips'                         => $params->hips,
        ':thigh'                        => $params->thigh,
        ':crotch'                       => $params->crotch,
        ':knee'                         => $params->knee,
        ':leg_length'                   => $params->leg_length,
        ':vest_length_front'            => $params->vest_length_front,
        ':vest_length_back'             => $params->vest_length_back,
        ':cuff'                         => $params->cuff,
        ':watch_hand'                   => $params->watch_hand,
        ':calf'                         => $params->calf,
        ':coat_sleeve_left'             => $params->coat_sleeve_left,
        ':coat_sleeve_right'            => $params->coat_sleeve_right
    );
    $pre->execute( $values );
}

function saveCustomerRentalMeasurements( $params, $customer_id, $con ) {
    $sql = "insert into `tblCustomerRentalMeasurements` (`customer_id`,
                                                    `coat_size`,
                                                    `coat_length`,
                                                    `coat_sleeve`,
                                                    `shirt_size`,
                                                    `pant_waist`,
                                                    `pant_outseam`,
                                                    `shoe_size`,
                                                    `date_created`) values (
                                                    :customer_id,
                                                    :coat_size,
                                                    :coat_length,
                                                    :coat_sleeve,
                                                    :shirt_size,
                                                    :pant_waist,
                                                    :pant_outseam,
                                                    :shoe_size,
                                                    now())";
    $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
    $values = array(
        ':customer_id'      => $customer_id,
        ':coat_size'        => $params->coat_size,
        ':coat_length'      => $params->coat_length,
        ':coat_sleeve'      => $params->coat_sleeve,
        ':shirt_size'       => $params->shirt_size,
        ':pant_waist'       => $params->waist,
        ':pant_outseam'     => $params->outseam,
        ':shoe_size'        => $params->shoe_size
    );
    $pre->execute( $values );
}

function transformStatusId( $from, $to, $order_id ) { 
    // transform form wintux to POS
    if ( $from == 'Wintux' && $to == 'POS' ) {

    }

    // transform form wintux to LFA Hub
    else if ( $from == 'Wintux' && $to == 'Hub' ) {

    }

    // transform form POS to Wintux
    else if ( $from == 'POS' && $to == 'Wintux' ) {

    }

    // transform form POS to LFA Hub
    else if ( $from == 'POS' && $to == 'Hub' ) {

    }

    // transform form LFA Hub to Wintux
    else if ( $from == 'Hub' && $to == 'Wintux' ) {

    }

    // transform form LFA Hub to POS
    else if ( $from == 'Hub' && $to == 'POS' ) {
        
    }
}

function validateOrderParameters( $params ) {
    $errors = array();

    if( strlen( $params->paymentToken ) == 0 && $params->paymentMethod == 'Stripe' ) 
        array_push( $errors, 'There was an error processing the payment. Make sure the payment details are correct.' );

    if( strlen($params->customerName) < 3 ) 
        array_push( $errors, 'The customer first name is required and must be valid.' );

    if( strlen($params->customerLastName) < 3 ) 
        array_push( $errors, 'The customer last name is required and must be valid.' );

    if( strlen($params->customerGender) < 1 ) 
        array_push( $errors, 'The customer gender is required and must be valid.' );

    $arr_birthdate = explode( '-', $params->customerBirthdate );
    if( !checkdate($arr_birthdate[1], $arr_birthdate[2], $arr_birthdate[0]) ) 
        array_push( $errors, 'The customer birth date is required and must be valid.' );

    if( strlen($params->customerAddressLine1) < 3 ) 
        array_push( $errors, 'The customer address line 1 is required and must be valid.' );

    if( strlen($params->customerState) < 2 ) 
        array_push( $errors, 'The customer address state is required and must be valid.' );

    if( strlen($params->customerZipcode) < 5 ) 
        array_push( $errors, 'The customer address zip code is required and must be valid.' );
    
    preg_match('/.+@.+/', $params->customerEmail, $matches);
    if( count( $matches ) == 0 ) {
        array_push( $errors, 'The customer email is required and must be valid.' );
    }

    preg_match('/^[+]*[(]{0,1}[0-9]{1,4}[)]{0,1}[-\s\.0-9]*$/', $params->customerPhone, $matches);
    if( count( $matches ) == 0 ) {
        array_push( $errors, 'The customer phone is required and must be valid.' );
    }

    preg_match('/^[0-9]{1}$/', $params->customerMeasurements->heightFeet, $matches);
    if( count( $matches ) == 0 ) {
        array_push( $errors, 'The height - feet is required and must be valid.' );
    }
    if( !( intval( $params->customerMeasurements->heightInches ) >= 0 &&
        intval( $params->customerMeasurements->heightInches ) <= 11 ) ) {
        array_push( $errors, 'The height - inches is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->neck ) ) {
        array_push( $errors, 'The neck measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->shoulder ) ) {
        array_push( $errors, 'The shoulder measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->sleeveLengthLeft ) ) {
        array_push( $errors, 'The sleeve length left measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->sleeveLengthRight ) ) {
        array_push( $errors, 'The sleeve length right measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->coatSleeveLengthLeft ) ) {
        array_push( $errors, 'The coat sleeve length left measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->coatSleeveLengthRight ) ) {
        array_push( $errors, 'The coat sleeve length right measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->bicep ) ) {
        array_push( $errors, 'The bicep measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->wrist ) ) {
        array_push( $errors, 'The wrist measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->chest ) ) {
        array_push( $errors, 'The chest measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->stomach ) ) {
        array_push( $errors, 'The stomach measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->jacketLengthFront ) ) {
        array_push( $errors, 'The jacket length front measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->jacketLengthBack ) ) {
        array_push( $errors, 'The jacket length back measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->waist ) ) {
        array_push( $errors, 'The waist measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->hips ) ) {
        array_push( $errors, 'The hips measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->thigh ) ) {
        array_push( $errors, 'The thigh measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->crotch ) ) {
        array_push( $errors, 'The crotch measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->knee ) ) {
        array_push( $errors, 'The knee measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->legLength ) ) {
        array_push( $errors, 'The leg length measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->vestFront ) ) {
        array_push( $errors, 'The vest front measurement is required and must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->vestBack ) ) {
        array_push( $errors, 'The vest back measurement is required and must be valid.' );
    }
    if( strlen( $params->customerMeasurements->calf ) > 0 && !is_numeric( $params->customerMeasurements->calf ) ) {
        array_push( $errors, 'The calf measurement must be valid.' );
    }
    if( !is_numeric( $params->customerMeasurements->cuff ) ) {
        array_push( $errors, 'The cuff measurement is required and must be valid.' );
    }
    if( strtolower( $params->customerMeasurements->watchHand ) != 'right' &&
        strtolower( $params->customerMeasurements->watchHand ) != 'left' &&
        strtolower( $params->customerMeasurements->watchHand ) != 'none' ) {
        array_push( $errors, 'The watch hand is required and must be valid.' );
    }

    if( strlen( $params->rep ) == 0 ) {
        array_push( $errors, 'The representative code is required and must be valid.' );
    }
    if( !is_numeric( $params->subtotal ) ) {
        array_push( $errors, 'There is something wrong with the order subtotal.' );
    }
    if( !is_numeric( $params->tax ) ) {
        array_push( $errors, 'There is something wrong with the order tax.' );
    }
    if( !is_numeric( $params->total ) ) {
        array_push( $errors, 'There is something wrong with the order total.' );
    }
    if( !is_numeric( floatval( $params->payment_amount ) ) ) {
        array_push( $errors, 'There is something wrong with the order payment amount.' );
    }
    // if( floatval( str_replace(',', '', $params->paymentAmount ) ) < round( floatval( $params->total ) * 0.25, 2 ) ) {
    //     array_push( $errors, 'There is a minimum payment amount of 25% of the order. ' );
    // }
    if( $params->paymentMethod != 'Stripe' && $params->paymentConfirmationNumber == '' ) {
        array_push( $errors, 'The payment confirmation number is required.' );
    }
    if( strlen( $params->store ) == 0 ) {
        array_push( $errors, 'The store is required.' );
    }

    return $errors;
}

function sendEmail ( $type, $user_full_name, $to_email, $order_id, $status = '' ) {

    $mail = new PHPMailer(true);
    try {
        // Server settings
        // $mail->SMTPDebug = 2;                                 // Enable verbose debug output
        $mail->isSMTP();                                      // Set mailer to use SMTP
        $mail->Host = 'mail.leonardofifthavenue.com';         // Specify main and backup SMTP servers
        $mail->SMTPAuth = false;                              // Enable SMTP authentication
        // $mail->Username = 'user@example.com';                 // SMTP username
        // $mail->Password = 'secret';                           // SMTP password
        // $mail->SMTPSecure = 'tls';                            // Enable TLS encryption, `ssl` also accepted
        $mail->Port = 26;                                     // TCP port to connect to

        // Recipients
        $mail->setFrom('customerservice@leonardofifthavenue.com', 'LFA - Made to Measure');
        $mail->addAddress($to_email, $user_full_name);     // Add a recipient

        // Attachments
        // $mail->addAttachment('/var/tmp/file.tar.gz');         // Add attachments
        // $mail->addAttachment('/tmp/image.jpg', 'new.jpg');    // Optional name

        // Content
        $mail->isHTML(true);                                  // Set email format to HTML

        switch( $type ) {
            case 'lcs.neworder':
                mailBodyLCSNewOrder( $mail, $user_full_name, $order_id, 'http://dev.leonardofifthavenue.com/m2mkiosk/admin/#/orders' );
                break;
            case 'admin.orderapproved':
                mailBodyAdminNewOrder( $mail, $user_full_name, $order_id, 'http://dev.leonardofifthavenue.com/m2mkiosk/admin/#/orders' );
                break;
            case 'client.neworder':
                mailBodyClientNewOrder( $mail, $user_full_name, $order_id, 'http://dev.leonardofifthavenue.com/m2mkiosk/pay' );
                break;
            case 'client.statuschange':
                mailBodyClientOrderStatusChange( $mail, $user_full_name, $order_id, 'http://dev.leonardofifthavenue.com/m2mkiosk/pay', $status );
                break;
        }

        // Send
        $mail->send();
        return 'Message has been sent';
    }
    catch ( Exception $ex ) {
        return 'Message could not be sent. Mailer Error: ' . $mail->ErrorInfo;
    }
}

function mailBodyLCSNewOrder( &$mail, $user_full_name, $order_id, $btn_link ) {
    $mail->Subject = 'Made to Measure Order #' . $order_id . ' waiting for approval';
    $mail->Body    = file_Get_contents('./email_templates/lcs_new_order.txt');

    $mail->Body    = str_replace('*|MC:SUBJECT|*', $mail->Subject, $mail->Body);
    $mail->Body    = str_replace('*|MC_PREVIEW_TEXT|*', $mail->Subject, $mail->Body);
    $mail->Body    = str_replace('*|FNAME|*', $user_full_name, $mail->Body);
    $mail->Body    = str_replace('*|ORDER_ID|*', $order_id, $mail->Body);
    $mail->Body    = str_replace('*|UPDATE_PROFILE|*', $btn_link, $mail->Body);
    $mail->Body    = str_replace('*|CURRENT_YEAR|*', date("Y"), $mail->Body);
}

function mailBodyAdminNewOrder( &$mail, $user_full_name, $order_id, $btn_link ) {
    $mail->Subject = 'Made to Measure Order #' . $order_id . ' has been approved by designer';
    $mail->Body    = file_Get_contents('./email_templates/admin_new_order.txt');

    $mail->Body    = str_replace('*|MC:SUBJECT|*', $mail->Subject, $mail->Body);
    $mail->Body    = str_replace('*|MC_PREVIEW_TEXT|*', $mail->Subject, $mail->Body);
    $mail->Body    = str_replace('*|FNAME|*', $user_full_name, $mail->Body);
    $mail->Body    = str_replace('*|ORDER_ID|*', $order_id, $mail->Body);
    $mail->Body    = str_replace('*|UPDATE_PROFILE|*', $btn_link, $mail->Body);
    $mail->Body    = str_replace('*|CURRENT_YEAR|*', date("Y"), $mail->Body);
}

function mailBodyClientNewOrder( &$mail, $user_full_name, $order_id, $btn_link ) {
    $mail->Subject = 'Thank you for your Made to Measure Order!';
    $mail->Body    = file_Get_contents('./email_templates/customer_new_order.txt');

    $mail->Body    = str_replace('*|MC:SUBJECT|*', $mail->Subject, $mail->Body);
    $mail->Body    = str_replace('*|MC_PREVIEW_TEXT|*', $mail->Subject, $mail->Body);
    $mail->Body    = str_replace('*|FNAME|*', $user_full_name, $mail->Body);
    $mail->Body    = str_replace('*|ORDER_ID|*', $order_id, $mail->Body);
    $mail->Body    = str_replace('*|UPDATE_PROFILE|*', $btn_link, $mail->Body);
    $mail->Body    = str_replace('*|CURRENT_YEAR|*', date("Y"), $mail->Body);
}

function mailBodyClientOrderStatusChange( &$mail, $user_full_name, $order_id, $btn_link, $status ) {
    $mail->Subject = 'Made to Measure Order #' . $order_id . ' status update';
    $mail->Body    = file_Get_contents('./email_templates/customer_status_changed.txt');

    $mail->Body    = str_replace('*|MC:SUBJECT|*', $mail->Subject, $mail->Body);
    $mail->Body    = str_replace('*|MC_PREVIEW_TEXT|*', $mail->Subject, $mail->Body);
    $mail->Body    = str_replace('*|FNAME|*', $user_full_name, $mail->Body);
    $mail->Body    = str_replace('*|ORDER_ID|*', $order_id, $mail->Body);
    $mail->Body    = str_replace('*|STATUS|*', $status, $mail->Body);
    $mail->Body    = str_replace('*|UPDATE_PROFILE|*', $btn_link, $mail->Body);
    $mail->Body    = str_replace('*|CURRENT_YEAR|*', date("Y"), $mail->Body);
}

$app->run();
// $cron->run();