var rptCuadre = Vue.component("m2m-admin-rpt-cuadre", {
    template: `<div>
        <m2m-admin-date-picker @dateChanged="onChangeDate" @load="onLoad"></m2m-admin-date-picker>
        <v-divider></v-divider>

        <v-data-table :headers="headers" :items="items" class="elevation-1" :rows-per-page-items="this.paginationOptions">
            <template slot="items" slot-scope="props">
                <td>{{ props.item.paymentMethod }}</td>
                <td class="text-xs-right">{{ formatMoney( props.item.total ) }}</td>
            </template>
            <template slot="footer">
                <tr>
                    <td class="body-2">
                        Totals
                    </td>
                    <td class="text-xs-right body-2">{{ formatMoney( this.sumTotal ) }}</td>
                </tr>
            </template>
        </v-data-table>
    </div>`,
    data: function() {
        return { 
            headers: [
                { text: 'Payment Method', value: 'paymentMethod', sortable: false, align: 'left', width: '70%' },
                { text: 'Total', value: 'total', sortable: false, align: 'right', width: '30%' },
            ],
            items: [],
            dates: [],
            paginationOptions: [ 25, 50, { "text": "$vuetify.dataIterator.rowsPerPageAll", "value": -1 } ],
        }
    },
    created () {
        serverBus.$emit('changePageTitle', 'Report: Cuadre');
    },
    mounted: function() {
        //this.loadDetails();
    },
    computed: {
        sumTotal: function() {
            let grandTotal = 0;
            this.items.forEach(item => {
                grandTotal += parseFloat(item.total);
            });

            return grandTotal;
        },
    },
    methods: {
        loadDetails: function() {
            axios
            .get( app.dataUrl + '?action=admin.rpt.cuadre&date=' + this.dates.join(';'), { })
            .then( response => { 
                if(response.data.status == false)
                    return;

                if(!response.data.status && response.data.signout) {
                    app.signOut();
                }
                if(response.data.token) {
                    app.token = response.data.token;
                    localStorage.setItem('m2m-admin-token', response.data.token);
                }

                this.items = response.data.data;
            });  
        },
        onLoad: function(value) {
            this.dates = value.split(';');
            this.loadDetails();
        },
        onChangeDate: function(value) {
            this.dates = value.split(';');
            this.loadDetails();
        },
        formatMoney: function(n, c, d, t) {
            var c = isNaN(c = Math.abs(c)) ? 2 : c,
              d = d == undefined ? "." : d,
              t = t == undefined ? "," : t,
              s = n < 0 ? "-" : "",
              i = String(parseInt(n = Math.abs(Number(n) || 0).toFixed(c))),
              j = (j = i.length) > 3 ? j % 3 : 0;
          
            return '$' + s + (j ? i.substr(0, j) + t : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.abs(n - i).toFixed(c).slice(2) : "");
        },
    }
});