Vue.component('m2m-store-login', {
    template: `
    <div class="text-xs-center">
        <v-dialog v-model="show" width="500" persistent>
            <v-card>
                <v-card-title>
                    <p class="title">Select Store</p>
                </v-card-title>
                <v-card-text>
                    <v-alert :value="showAlert" color="error" icon="warning">{{ this.alertMessage }}</v-alert>
                    <v-select :items="stores" v-model="selectValue" label="Store"></v-select>
                    <v-text-field label="Password" v-model="storePassword" type="password"></v-text-field>
                </v-card-text>

                <v-divider></v-divider>

                <v-card-actions>
                    <v-spacer></v-spacer>
                    <v-btn color="primary" flat @click="loginStore">
                        Login
                    </v-btn>
                </v-card-actions>
            </v-card>
        </v-dialog>
    </div>
`,
    props: {
        show: Boolean
    },
    data() {
        return {
            stores: [
                { text: 'San Juan', value: 1 },
                { text: 'Caguas', value: 2 },
                { text: 'Bayamón', value: 3 },
                { text: 'Mayaguez', value: 4 },
                { text: 'Ponce', value: 5 }
            ],
            selectValue: '',
            storePassword: '',
            showAlert: false,
            alertMessage: '',
        }
    },
    computed: {
        form: function () {
            return {
                storeId: this.selectValue,
                password: this.storePassword
            }
        },
    },
    methods: {
        loginStore: function() {
            this.showAlert = false;
            this.alertMessage = '';
            
            if( this.selectValue != '' && this.storePassword != '') {
                let axiosOptions = {
                    method: 'POST',
                    url: app.dataUrl,
                    data: { 'form': this.form },
                    params: {
                        'action': 'valstr'
                    },
                    headers: {
                        'Authorization': app.authKey
                    },
                    json: true
                };

                axios(axiosOptions)
                .then( response => { 
                    // If everything is ok, go to thankyou page
                    if( response.data.status == true ) {
                        var obj = { value: this.selectValue, timeStamp: new Date().getTime() }
                        localStorage.setItem("m2m-selected-store", JSON.stringify(obj));
                        app.store.store = obj.value;

                        this.storeDialog = false;
                        this.storePassword = '';
                        this.selectValue = '';
                        this.$emit('loggedin');
                        return;
                    }

                    this.alertMessage = response.data.message;
                    this.showAlert = true;
                })
                .catch(function (error) {
                    console.log(error);
                    this.processing = false;
                });
            }
            else
                alert('You must select a store and enter the password before login in.');
        },
    }
})