Vue.component("m2m-rep-list", {
    template: `
    <div>
        <m2m-loading :loading="this.loading">
            <v-card flat>
                <v-card-title primary-title>
                    <v-layout row>
                        <v-flex xs8>
                            <div class="title">{{ this.repType }}s</div>
                        </v-flex>
                        <v-flex xs4 class="text-xs-right">
                            <v-dialog v-model="dialog" fullscreen hide-overlay transition="dialog-bottom-transition">
                                <v-btn small right class="ma-0" slot="activator" v-if="isEditable">Add New</v-btn>
                                <v-card dark>
                                    <v-toolbar dark color="primary">
                                        <v-btn icon dark @click="closeDialog">
                                            <v-icon>close</v-icon>
                                        </v-btn>
                                        <v-toolbar-title>{{ this.dialogTitle }}</v-toolbar-title>
                                        <v-spacer></v-spacer>
                                        <v-toolbar-items>
                                            <v-btn dark flat @click="saveStylist">Save</v-btn>
                                        </v-toolbar-items>
                                    </v-toolbar>
                                    <v-subheader>General Information</v-subheader>
                    
                                    <v-form ref="form" lazy-validation class="pa-3">
                                        <v-text-field label="Name" v-model="stylistName"></v-text-field>
                                        <v-text-field label="Code" v-model="stylistCode" :disabled="isEdit"></v-text-field>
                                        <v-checkbox v-model="stylistActive" label="Active"></v-checkbox>
                                    </v-form>
                                </v-card>
                            </v-dialog>
                        </v-flex>
                    </v-layout>
                </v-card-title>
                <v-card-text>
                    <v-layout row v-if="allowFilter">
                        <v-flex xs8></v-flex>
                        <v-flex xs4>
                            <v-text-field v-model="search" append-icon="search" label="Filter" single-line hide-details></v-text-field>
                        </v-flex>
                    </v-layout>
                    <v-data-table :headers="this.headers" :items="this.stylists" :search="search" :hide-actions="!allowPagination">
                        <template slot="items" slot-scope="props">
                            <td>{{ props.item.repName }}</td>
                            <td>{{ props.item.repCode }}</td>
                            <td>
                                <v-checkbox v-model="props.item.active" primary hide-details @change="toggleStylist(props.item)" :disabled="checkboxStatus"></v-checkbox>
                            </td>
                            <td class="justify-center layout px-0" v-if="isEditable">
                                <v-icon small class="mr-2" @click="editStylist(props.item)">
                                    edit
                                </v-icon>
                            </td>
                        </template>
                    </v-data-table>
                </v-card-text>
            </v-card>
        </m2m-loading>
    </div>
    `,
    props: {
        editable: Boolean,
        type: String,
        allowFilter: Boolean,
        allowPagination: Boolean
    },
    data: function() {
        return { 
            loading: false,
            dialog: false,
            dialogTitle: '',
            stylists: [],
            search: '',
            stylistName: '',
            stylistCode: '',
            stylistActive: true,
            isEdit: false,
        }
    },
    computed: {
        headers: function() {
            if(this.isEditable) {
                return [
                    { text: 'Name', value: 'repName', width: '50%', sortable: true },
                    { text: 'Code', value: 'repCode', width: '30%', sortable: true },
                    { text: 'Active', value: 'active', width: '10%', sortable: false },
                    { text: 'Edit', value: '', width: '10%', sortable: false }
                ];
            }

            return [
                { text: 'Name', value: 'repName', width: '60%', sortable: true },
                { text: 'Code', value: 'repCode', width: '30%', sortable: true },
                { text: 'Active', value: 'active', width: '10%', sortable: false },
            ];
        },
        isEditable: function() {
            if (this.editable == null || this.editable == undefined)
                return false;
            
            return this.editable;
        },
        repType: function() {
            if (this.type == null || this.type == undefined) 
                return 'Stylist';

            return this.type;
        },
        checkboxStatus: function() {
            return !this.isEditable; 
        },
    },
    created () {
        
    },
    mounted: function() {
        this.load();
    },
    methods: {
        load: function() {
            this.dialogTitle = 'New ' + this.repType;
            this.loading = true;
            axios
            .get( app.dataUrl + '?action=replist/' + this.repType, { })
                .then( response => { 
                    if( response.data.status ) {
                        this.stylists = response.data.data;
                        this.stylists.forEach(s => {
                            s.active = s.active == 1 ? true : false;
                        });
                    }

                    this.loading = false;
                });
        },
        toTitleCase: function (str) {
            return str.replace(/\-+/g, ' ').replace(/\w\S*/g, function(txt){
                return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();
            });
        },
        editStylist: function(user) {
            this.isEdit = true;
            
            this.dialogTitle = 'Edit ' + this.repType;
            this.stylistName = user.repName;
            this.stylistCode = user.repCode;
            this.stylistActive = user.active;
            this.dialog = true;
        },
        saveStylist: function() {
            this.loading = true;

            let formData = {
                repName: this.stylistName,
                repCode: this.stylistCode,
                active: this.stylistActive,
                type: this.repType,
                isEdit: this.isEdit
            };

            let axiosOptions = {
                method: 'POST',
                url: app.dataUrl,
                data: { 'form': formData },
                params: {
                    'action': 'admin.newrep'
                },
                json: true
            };

            axios(axiosOptions)
            .then( response => { 
                // If everything is ok, go to thankyou page
                if( !response.data.status ) {
                    // show error dialog
                    alert(response.data.error );
                    return;
                }
                else if(response.data.status == 'false' ) {
                    // show error dialog
                    alert(response.data.message );
                    return;
                }

                // Add/Edit the new stylist to the table
                if ( this.isEdit ) {
                    let editStylist = this.stylists.find(stylist => {
                        return stylist.repCode == formData.repCode;
                    });

                    editStylist.repName = formData.repName;
                    editStylist.active = formData.active;
                }
                else {
                    this.stylists.push(
                        { repName: formData.repName, repCode: formData.repCode, active: formData.active }
                    );
                }

                this.dialog = false;
                this.stylistName = this.stylistCode = '';
                this.stylistActive = true;
                this.isEdit = false;

                this.loading = false;
            })
            .catch(function (error) {
                console.log(error);
                this.loading = false;
            });
        },
        toggleStylist: function(stylist) {
            let formData = {
                repCode: stylist.repCode,
                active: stylist.active
            };

            let axiosOptions = {
                method: 'POST',
                url: app.dataUrl,
                data: { 'form': formData },
                params: {
                    'action': 'admin.togglerep'
                },
                json: true
            };

            axios(axiosOptions)
            .then( response => { 
                // If everything is ok, go to thankyou page
                if( !response.data.status ) {
                    // show error dialog
                    alert(response.data.error );
                    return;
                }
                else if(response.data.status == 'false' ) {
                    // show error dialog
                    alert(response.data.message );
                    return;
                }


                let editStylist = this.stylists.find(s => {
                    return s.userName == formData.userName;
                });

                editStylist.active = formData.active;
            })
            .catch(function (error) {
                console.log(error);
            });
        },
        closeDialog: function() {
            this.dialog = false;
            
            this.dialogTitle = 'New ' + this.repType;
            this.stylistName = this.stylistCode = '';
            this.stylistActive = true;
            this.isEdit = false;
        },
    }
});