var orders = Vue.component("m2m-admin-orders", {
    template: `<div>
    <v-card flat>
        <v-card-title>
            <v-overflow-btn :items="this.orderStatusList" label="Change Order Status" target="#orders" v-if="this.selected.length > 0" flat dark small class="mt-0 pt-0"></v-overflow-btn>
            <v-spacer></v-spacer>
            <v-text-field v-model="search" append-icon="search" label="Filter" single-line hide-details></v-text-field>
        </v-card-title>
    </v-card>
    <v-data-table :headers="this.headers" :items="this.orderList" :rows-per-page-items="this.paginationOptions" v-model="selected"
      :search="search" item-key="orderId" :pagination.sync="pagination">
      <template slot="items" slot-scope="props">
        <!--<td>
          <v-checkbox v-model="props.selected" primary hide-details></v-checkbox>
        </td>-->
        <td>{{ props.item.orderId }}</td>
        <td>{{ props.item.orderDate }}</td>
        <td>{{ toTitleCase(props.item.orderStatus) }}</td>
        <td>{{ props.item.customerName }}</td>
        <td>{{ props.item.customerPhone }}</td>
        <td class="text-xs-right">{{ formatMoney(props.item.total) }}</td>
        <td class="text-xs-right">{{ formatMoney(props.item.balance) }}</td>
        <td class="justify-center layout px-0">
            <v-icon small class="mr-2" @click="openOrder(props.item)">
                remove_red_eye
            </v-icon>
        </td>
      </template>
    </v-data-table>
</div>`,
    data: function() {
        return { 
            selected: [],
            headers: [
                { text: 'Order #', value: 'orderId', sortable: true, align: 'left' },
                { text: 'Order Date', value: 'orderDate', sortable: true, align: 'left' },
                { text: 'Status', value: 'orderStatus', sortable: true, align: 'left' },
                { text: 'Customer Name', value: 'customerName', sortable: true, align: 'left' },
                { text: 'Customer Phone', value: 'customerPhone', sortable: false, align: 'left' },
                { text: 'Order Total', value: 'total', sortable: true, align: 'right' },
                { text: 'Balance', value: 'balance', sortable: true, align: 'right' },
                { text: 'View Details', value: 'name', sortable: false }
            ],
            pagination: {
                sortBy: 'orderId',
                descending: true
            },
            orderList: [],
            paginationOptions: [ 25, 50, { "text": "$vuetify.dataIterator.rowsPerPageAll", "value": -1 } ],
            search: '',
            orderStatusList: [ 'Received', 'Awaiting Payment', 'Processing', 'Shipped', 'Completed', 'Cancelled', 'Refunded' ]
        }
    },
    created () {
        serverBus.$emit('changePageTitle', 'Order List');
        serverBus.$on('signedIn', () => {
            this.loadOrders();
        });
    },
    mounted: function() {
        this.loadOrders();
    },
    methods: {
        loadOrders: function() {
            axios
                .get( app.dataUrl + '?action=admin.orderlist', { })
                .then( response => { 
                    if(!response.data.status && response.data.signout) {
                        app.signOut();
                    }
                    if(response.data.token) {
                        app.token = response.data.token;
                        localStorage.setItem('m2m-admin-token', response.data.token);
                    }

                    this.orderList = response.data.data;
                });  
        },
        openOrder: function(item) {
            app.store.orderToView = item.orderId;
            this.$router.push('/orderdetails');
        },
        formatMoney: function(n, c, d, t) {
            var c = isNaN(c = Math.abs(c)) ? 2 : c,
              d = d == undefined ? "." : d,
              t = t == undefined ? "," : t,
              s = n < 0 ? "-" : "",
              i = String(parseInt(n = Math.abs(Number(n) || 0).toFixed(c))),
              j = (j = i.length) > 3 ? j % 3 : 0;
          
            return '$' + s + (j ? i.substr(0, j) + t : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.abs(n - i).toFixed(c).slice(2) : "");
        },
        toTitleCase: function (str) {
            return str.replace(/\-+/g, ' ').replace(/\w\S*/g, function(txt){
                return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();
            });
        },
    }
});