Vue.component('m2m-mods', {
    props: ['typeId','modCategories'],
    template: `
                <v-layout wrap>
                    <v-flex d-flex xs4>
                        <ul class="categories">
                            <li v-for="cat in modCategories" v-bind:key="cat.mod_id" :class="cat.selected == true || cat.selected == 1 ? 'selected' : ''" @click="loadOptions(cat)">
                                <img :src="cat.image_url" width="60" height="60" />
                            </li>
                        </ul>
                    </v-flex>
                    <v-flex d-flex xs8>
                        <ul class="sub-categories">
                            <li v-for="opt in modOptions" v-bind:key="opt.mod_option_id" @click="setOption(opt)" :class="opt.selected == true || opt.selected == 1 ? 'selected' : ''">
                                <img :src="opt.image_url" width="80" height="80" />
                            </li>
                        </ul>
                    </v-flex>
                </v-layout>
            `,
    created () {
        serverBus.$on('editModifications', () => {
            this.applyModifications();
        });
    },
    mounted: function() {

    },
    data: function() {
        return {
            modOptions: null
        }
    },
    methods: {
        loadOptions: function( cat ) {
            axios
                .get( app.dataUrl + '?action=modopt&c=' + cat.mod_id, { headers: { Authorization: app.authKey } })
                .then( response => { 
                    this.modOptions = response.data.data;
                    this.cleanSelected();
                    var category = this.modCategories.find(function(item) {
                        return item.mod_id == cat.mod_id;
                    });
                    
                    category.selected = true;

                    var selectedOpt = app.store.modifications.find(function(item) {
                        return item.catId == category.mod_id;
                    });

                    if ( selectedOpt != null ) {
                        var opt = this.modOptions.find(function(item) {
                            return item.mod_option_id == selectedOpt.optId
                        });

                        opt.selected = true;
                    }
                });
        },
        setOption: function( opt ) {
            var category = this.modCategories.find(function(item) {
                return item.selected == true;
            });
            var exist = app.store.modifications.find(function(item) {
                return item.catId == category.mod_id;
            });

            if ( exist ) {

                if ( exist.optId == opt.mod_option_id ) 
                    return;

                exist.optId = opt.mod_option_id;
                exist.option = opt.name.replace(',', '').replace(/\s+/g, '-').toLowerCase();
                exist.optionImage = opt.image_url;
            }
            else {
                app.store.modifications.push({
                    catId: category.mod_id,
                    category: category.name.replace(/\s+/g, '-').toLowerCase(),
                    optId: opt.mod_option_id,
                    option: opt.name.replace(',', '').replace(/\s+/g, '-').toLowerCase(),
                    optionImage: opt.image_url,
                    description: ''
                });
            }

            this.cleanOptSelected();
            opt.selected = true;
            this.applyModifications();
        },
        cleanSelected: function() {
            for( var i = 0; i < this.modCategories.length; i++ ) {
                this.modCategories[i].selected = false;
            }
        },
        cleanOptSelected: function() {
            for( var i = 0; i < this.modOptions.length; i++ ) {
                this.modOptions[i].selected = false;
            }
        },
        applyModifications: function() {
            // Get all values
            if ( app.store.typeId == 1 ) { // Coat
                // Hide all layers
                $('#lcm-graphic.lcm-coat .lcm-customizer').css('display', 'none');

                // Show the selected options
                var buttons = app.store.modifications.find(function(item) {
                    return item.category == "buttons";
                });
                buttons = (buttons == undefined ? { option: '' } : buttons);
                $('#lcm-graphic.lcm-coat .lcm-customizer-coat-' + buttons.option).css('display', 'inline');

                var lapel = app.store.modifications.find(function(item) {
                    return item.category == "lapel";
                });
                lapel = (lapel == undefined ? { option: '' } : lapel);
                var stitches = app.store.modifications.find(function(item) {
                    return item.category == "handstitches";
                });
                stitches = (stitches == undefined ? { option: '' } : stitches);
                $('#lcm-graphic.lcm-coat .lcm-customizer-coat-' + lapel.option).css('display', 'inline');
                $('#lcm-graphic.lcm-coat .lcm-customizer-coat-' + lapel.option + (stitches.option == 'handstitches' ? '-handstitches' : '')).css('display', 'inline');

                var sleeveButtons = app.store.modifications.find(function(item) {
                    return item.category == "sleeve-buttons";
                });
                sleeveButtons = (sleeveButtons == undefined ? { option: '' } : sleeveButtons);
                $('#lcm-graphic.lcm-coat .lcm-customizer-coat-' + sleeveButtons.option).css('display', 'inline');

                var pockets = app.store.modifications.find(function(item) {
                    return item.category == "pockets";
                });
                pockets = (pockets == undefined ? { option: '' } : pockets);
                var pocketType = app.store.modifications.find(function(item) {
                    return item.category == "pocket-type";
                });
                pocketType = (pocketType == undefined ? { option: 'pipped' } : pocketType);
                $('#lcm-graphic.lcm-coat .lcm-customizer-coat-' + pockets.option).css('display', 'inline');
                $('#lcm-graphic.lcm-coat .lcm-customizer-coat-' + pockets.option + (stitches.option == 'handstitches' ? '-handstitches' : '')).css('display', 'inline');
                $('#lcm-graphic.lcm-coat .lcm-customizer-coat-' + pockets.option + (pocketType.option != '' ? ('-' + pocketType.option) : '')).css('display', 'inline');
                // $('#lcm-graphic.lcm-coat .lcm-customizer-coat-' + pockets.option + obj.modification.stitches + obj.modification.pocketType).css('display', 'inline');

                if ( 'lcm-customizer-coat-' + buttons.option == 'lcm-customizer-coat-single-1-button' || 'lcm-customizer-coat-' + buttons.option == 'lcm-customizer-coat-single-2-buttons' ) {
                    $('#lcm-graphic.lcm-coat .lcm-double-suit').css('display', 'none');
                    $('#lcm-graphic.lcm-coat .lcm-double').css('display', 'none');
                }
                else {
                    $('#lcm-graphic.lcm-coat .lcm-double-suit').css('display', 'inline');
                    $('#lcm-graphic.lcm-coat .lcm-single').css('display', 'none');
                }

                var lapelWidth = app.store.modifications.find(function(item) {
                    return item.category == "lapel-width";
                });
                lapelWidth = (lapelWidth == undefined ? { option: 'regular' } : lapelWidth);
                switch( 'lcm-customizer-coat-' + lapelWidth.option ) {
                    case 'lcm-customizer-coat-regular':
                        $('#lcm-graphic.lcm-coat .lcm-wider, #lcm-graphic.lcm-coat .lcm-thinner').css('display', 'none');
                        break;
                    case 'lcm-customizer-coat-wider':
                        $('#lcm-graphic.lcm-coat .lcm-regular, #lcm-graphic.lcm-coat .lcm-thinner').css('display', 'none');
                        break;
                    case 'lcm-customizer-coat-thinner':
                        $('#lcm-graphic.lcm-coat .lcm-regular, #lcm-graphic.lcm-coat .lcm-wider').css('display', 'none');
                        break;   
                }
            }
            else if ( app.store.typeId == "2" ) { // Pants
                let tape = app.store.modifications.find(function(item) {
                    return item.category == "satin-tape";
                });
                tape = (tape == undefined ? { option: '' } : tape);

                let pockets = app.store.modifications.find(function(item) {
                    return item.category == "back-pocket";
                });
                pockets = (pockets == undefined ? { option: '' } : pockets);
                
                // var tape = {
                //     base: $('#lcm-tab-satintape .lcm-option-wrapper.active .lcm-option-name a').attr('lcm-svg-base-class'),
                //     class: $('#lcm-tab-satintape .lcm-option-wrapper.active .lcm-option-name a').attr('lcm-svg-class'),
                //     option: $('#lcm-tab-satintape .lcm-option-wrapper.active .lcm-option-name a').attr('lcm-option')
                // };
                // var pockets = {
                //     base: $('#lcm-tab-backpocket .lcm-option-wrapper.active .lcm-option-name a').attr('lcm-svg-base-class'),
                //     class: $('#lcm-tab-backpocket .lcm-option-wrapper.active .lcm-option-name a').attr('lcm-svg-class'),
                //     option: $('#lcm-tab-backpocket .lcm-option-wrapper.active .lcm-option-name a').attr('lcm-option')
                // };

                // Hide all layers
                $('#lcm-graphic.lcm-pant .lcm-customizer').css('display', 'none');

                // Show the selected options
                $('#lcm-graphic.lcm-pant .lcm-customizer-pant-' + tape.option).css('display', 'inline');
                $('#lcm-graphic.lcm-pant .lcm-customizer-pant-' + pockets.option).css('display', 'inline');
            }
            else if ( app.store.typeId == "3" ) { // Shirt
                let body = app.store.modifications.find(function(item) {
                    return item.category == "shirt-body";
                });
                body = (body == undefined ? { option: '' } : body);

                let neck = app.store.modifications.find(function(item) {
                    return item.category == "neck-type";
                });
                neck = (neck == undefined ? { option: '' } : neck);

                let cuffs = app.store.modifications.find(function(item) {
                    return item.category == "cuffs";
                });
                cuffs = (cuffs == undefined ? { option: '' } : cuffs);

                let contrast = app.store.modifications.find(function(item) {
                    return item.category == "contrast";
                });
                contrast = (contrast == undefined ? { option: '' } : contrast);
                
                let pocket = app.store.modifications.find(function(item) {
                    return item.category == "pocket";
                });
                pocket = (pocket == undefined ? { option: '' } : pocket);

                let pleats = app.store.modifications.find(function(item) {
                    return item.category == "pleats";
                })

                // Hide all layers
                $('#lcm-graphic.lcm-shirt .lcm-customizer').css('display', 'none');

                // Show the selected options
                $('#lcm-graphic.lcm-shirt .lcm-customizer-shirt-' + body.option).css('display', 'inline');
                $('#lcm-graphic.lcm-shirt .lcm-customizer-shirt-' + neck.option).css('display', 'inline');
                $('#lcm-graphic.lcm-shirt .lcm-customizer-shirt-' + cuffs.option).css('display', 'inline');
                $('#lcm-graphic.lcm-shirt .lcm-customizer-shirt-' + contrast.option).css('display', 'inline');
                $('#lcm-graphic.lcm-shirt .lcm-customizer-shirt-' + pocket.option).css('display', 'inline');
                $('#lcm-graphic.lcm-shirt .lcm-customizer-shirt-' + pleats.option).css('display', 'inline');

                serverBus.$emit('applyShirtFabrics', this.type);
            }
            else if ( app.store.typeId == "4" ) { // Vest
                let type = app.store.modifications.find(function(item) {
                    return item.category == "type";
                });
                type = (type == undefined ? { option: '' } : type);
                
                let pocket = app.store.modifications.find(function(item) {
                    return item.category == "pockets";
                });
                pocket = (pocket == undefined ? { option: '' } : pocket);

                // Hide all layers
                $('#lcm-graphic.lcm-vest .lcm-customizer').css('display', 'none');

                // Show the selected options
                $('#lcm-graphic.lcm-vest .lcm-customizer-vest-' + type.option).css('display', 'inline');
                $('#lcm-graphic.lcm-vest .lcm-customizer-vest-' + pocket.option).css('display', 'inline');

                serverBus.$emit('applyVestFabrics', this.type);
            }
        }
    }
});