var designer = Vue.component('product-design', {
    props: ['typeId'],
    data: function () {
        return {
            svg: null,
            modCategories: null,
            fabricCategories: null,
            loading: false,
            snackbar: false,
            snackbarText: 'Item added to cart',
            addToCartButton: 'add_shopping_cart',
            missingModifications: [],
            missingFabrics: [],
            dialog: false,
            coutureDialog: false,
            couturePrice: 0.00,
            coutureBtn: 'Couture',
            coutureAddToCart: 'Add To Cart',
            coutureValidation: false
        }
    },
    template: `
    <v-app>
        <m2m-loading v-if="loading"></m2m-loading>
        <v-container fluid fill-width designer>
            <v-layout row wrap fluid>
                <v-flex xs3>
                    <p class="display-1 grey--text text-xs-center">Modifications</p>
                    <m2m-mods :modCategories="this.modCategories"></m2m-mods>
                </v-flex>
                <v-flex xs6>
                    <div v-html="svg" style="width:600px;margin:0 auto;">
                    </div>
                </v-flex>
                <v-flex xs3>
                    <p class="display-1 grey--text text-xs-center">Fabrics</p>
                    <m2m-fabrics :fabricCategories="this.fabricCategories"></m2m-fabrics>
                </v-flex>
            </v-layout>
        </v-container>
        <div>
            <v-layout row justify-center align-center style="position: fixed;bottom:0;right:0;">
                <v-btn color="green darken-1" @click="openCoutureDialog" dark>{{ coutureBtn }}</v-btn>
                <v-dialog v-model="coutureDialog" max-width="600px" persistent>
                    <v-card>
                        <v-card-title>
                            <span class="headline">Couture</span>
                        </v-card-title>

                        <v-card-text>
                            <v-alert :value="coutureValidation" color="error" icon="warning" outline dismissible>
                                All fields are required.
                            </v-alert>
                            <v-layout row>
                                <v-flex xs12>
                                    <v-text-field label="Price" prefix="$" v-model="couturePrice" @keydown="priceChange($event)"></v-text-field>
                                </v-flex>
                            </v-layout>
                            <v-layout row wrap>
                                <v-flex xs6 v-for="cat in this.modCategories" :key="cat.mod_id">
                                    <v-textarea name="input-7-1" :label="cat.name" value="" rows="2" v-model="cat.description"></v-textarea>
                                </v-flex>
                            </v-layout>
                        </v-card-text>

                        <v-divider></v-divider>

                        <v-card-actions>
                            <v-spacer></v-spacer>
                            <v-btn color="error" flat @click="coutureDialog = false">
                                Cancel
                            </v-btn>
                            <v-btn color="primary" flat @click="addCouture">
                                {{ coutureAddToCart }}
                            </v-btn>
                        </v-card-actions>
                    </v-card>
                </v-dialog>
                <v-btn color="green darken-1" @click="addToCart()" dark fab><v-icon>{{ addToCartButton }}</v-icon></v-btn>
            </v-layout>
        </div>
        <v-snackbar v-model="snackbar" color="success" :top="true" :multi-line="false" :timeout="3000">
            <div>
                <v-icon left class="pl-0 pr-0">done</v-icon>
                <span class="pl-0">{{ snackbarText }}</span>
            </div>
        </v-snackbar>
        <v-dialog v-model="dialog" scrollable  max-width="500">
            <v-card>
                <v-card-title class="headline red darken-1"><v-icon>error</v-icon> Missing Configuration</v-card-title>
                <v-divider></v-divider>
                <v-card-text>Your missing the following modifications and/or fabrics. Please finish the configuration and then add it to the cart.</v-card-text>
                <v-card-text style="height: 300px;" class="pt-0 pb-0">
                    <v-container fluid fill-width class="pt-0 pb-0">
                        <v-layout row wrap fluid>
                            <v-flex xs6>
                                <v-list subheader>
                                    <v-subheader>Modifications</v-subheader>

                                    <v-list-tile avatar v-for="mod in missingModifications" :key="mod">
                                        <v-list-tile-content>
                                            <v-list-tile-title>{{ toTitleCase(mod) }}</v-list-tile-title>
                                        </v-list-tile-content>
                                    </v-list-tile>
                                </v-list>
                            </v-flex>
                            <v-flex xs6>
                                <v-list subheader>
                                    <v-subheader>Fabrics</v-subheader>

                                    <v-list-tile avatar v-for="fab in missingFabrics" :key="fab">
                                        <v-list-tile-content>
                                            <v-list-tile-title>{{ toTitleCase(fab) }}</v-list-tile-title>
                                        </v-list-tile-content>
                                    </v-list-tile>
                                </v-list>
                            </v-flex>
                        </v-layout>
                    </v-container>
                </v-card-text>
                <v-divider></v-divider>
                <v-card-actions>
                    <v-btn color="grey darken-1" flat @click.native="dialog = false" style="margin-left:auto; margin-right:0;">Ok</v-btn>
                </v-card-actions>
            </v-card>
        </v-dialog>
    </v-app>`,
    created () {
        this.typeId = app.store.typeId;
        this.load();
    },
    watch: {
        // call again the method if the route changes
        '$route': 'load'
    },
    methods: {
        load: function() {
            this.loading = true;

            if (!app.store.isEdit) {
                app.store.modifications = [];
                app.store.fabrics = [];
            }
            else {
                this.addToCartButton = 'edit';
                this.couturePrice = app.store.cart[app.store.editItemIndex].price;
                this.coutureBtn = 'Edit Couture';
                this.coutureAddToCart = 'Save Changes';
            }

            axios
                .get( app.dataUrl + '?action=modcat&t=' + this.typeId, { headers: { Authorization: app.authKey } })
                .then( response => { 
                    this.svg = response.data.svg;
                    this.modCategories = response.data.data;
                    this.loading = false;

                    if( app.store.isEdit ) {
                        setTimeout(function() { 
                            serverBus.$emit('editModifications', this.type);
                        }, 1000);
                        
                        app.store.modifications.forEach(mod => {
                            var m = this.modCategories.find(function(item) {
                                return item.mod_id == mod.catId;
                            });
            
                            m.description = mod.description;
                        });
                    }
                });
            axios
                .get( app.dataUrl + '?action=fabcat&t=' + this.typeId, { headers: { Authorization: app.authKey } })
                .then( response => { 
                    this.fabricCategories = response.data.data;

                    if( app.store.isEdit ) {
                        setTimeout(function() { 
                            serverBus.$emit('editFabrics', this.type);
                        }, 1500);
                    }
                });

            app.pageStack.splice(2);
            app.pageStack[1].disabled = false;
            app.pageStack.push( {
                text: 'Designer',
                disabled: true,
                href: '#/designer'
            });
        },
        openCoutureDialog: function() {
            let validation = this.validate();
            if(!validation.valid) {
                this.missingModifications = validation.missingModifications;
                this.missingFabrics = validation.missingFabrics;
                this.dialog = true;
                this.coutureDialog = false;
                return;
            }

            this.coutureDialog = true;
        },
        addCouture: function() {
            let validation = this.validate() 
            let coutureValidation = this.validateCouture();
            if(!validation.valid) {
                this.missingModifications = validation.missingModifications;
                this.missingFabrics = validation.missingFabrics;
                this.dialog = true;
                return;
            }
            if ( !coutureValidation )
                return;

            app.store.modifications.forEach(mod => {
                var m = this.modCategories.find(function(item) {
                    return item.mod_id == mod.catId;
                });

                mod.description = m.description;
            });

            let svg = new Object();
            Object.assign(svg, $('#lcm-graphic'));
            $(svg).find("[style*='display:none']").remove();
            let item = {
                typeId:         app.store.typeId,
                productId:      app.store.productId,
                productName:    app.store.productName,
                price:          this.couturePrice, //app.store.price,
                qty:            app.store.qty,
                modifications:  app.store.modifications,
                fabrics:        app.store.fabrics,
                image:          svg[0].outerHTML,
                isCouture:      true
            };

            if ( app.store.isEdit ) { // Update the item
                app.store.cart[app.store.editItemIndex].modifications = item.modifications;
                app.store.cart[app.store.editItemIndex].fabrics = item.fabrics;
                app.store.cart[app.store.editItemIndex].image = item.image;
                app.store.cart[app.store.editItemIndex].isCouture = item.isCouture;
                app.store.cart[app.store.editItemIndex].price = this.couturePrice;
                this.snackbarText = 'Item updated successfully'
            }
            else { // Add new item to cart

                // Check if another item with the same configuration exists
                let cartItem = _.find(app.store.cart, function(i) {
                    return _.isEqual(i.modifications, item.modifications) && _.isEqual(i.fabrics, item.fabrics) && i.typeId == item.typeId && i.isCouture == item.isCouture;
                });

                // if an item exist with the same configuration, edit qty of cart item
                if (cartItem != undefined) 
                    cartItem.qty++;
                else                        // Else, add new item to the cart
                    app.store.cart.push(item);
                
                this.snackbarText = 'Item added to cart'                
            }

            this.snackbar = true;
            this.coutureDialog = false;
        },
        addToCart: function() {

            let validation = this.validate();
            if(!validation.valid) {
                this.missingModifications = validation.missingModifications;
                this.missingFabrics = validation.missingFabrics;
                this.dialog = true;
                return;
            }
            
            let svg = new Object();
            Object.assign(svg, $('#lcm-graphic'));
            $(svg).find("[style*='display:none']").remove();
            let item = {
                typeId:         app.store.typeId,
                productId:      app.store.productId,
                productName:    app.store.productName,
                price:          app.store.price,
                qty:            app.store.qty,
                modifications:  app.store.modifications,
                fabrics:        app.store.fabrics,
                image:          svg[0].outerHTML,
                isCouture:      false
            };

            switch (item.typeId) {
                case 1: // Coat
                    let cBody = item.fabrics.find(function(i) {
                        return i.fabric_option == 'coatbody';
                    });

                    if( cBody.fabric.substr(0, 1).toLowerCase() == 'd' )
                        item.price += 300.00;

                    break;
                case 2: // Pant
                    let pBody = item.fabrics.find(function(i) {
                        return i.fabric_option == 'pantbody';
                    });

                    if( pBody.fabric.substr(0, 1).toLowerCase() == 'd' )
                        item.price += 100.00;
                        
                    break;
                case 3: break; // Shirt
                case 4: // Vest
                    let vBody = item.fabrics.find(function(i) {
                        return i.fabric_option == 'vestbody';
                    });

                    if( vBody.fabric.substr(0, 1).toLowerCase() == 'd' )
                        item.price += 259.00;
                    break; 
            }

            if ( app.store.isEdit ) { // Update the item
                app.store.cart[app.store.editItemIndex].modifications = item.modifications;
                app.store.cart[app.store.editItemIndex].fabrics = item.fabrics;
                app.store.cart[app.store.editItemIndex].image = item.image;
                app.store.cart[app.store.editItemIndex].isCouture = item.isCouture;
                app.store.cart[app.store.editItemIndex].price = item.price;
                this.snackbarText = 'Item updated successfully';
            }
            else { // Add new item to cart

                // Check if another item with the same configuration exists
                let cartItem = _.find(app.store.cart, function(i) {
                    return _.isEqual(i.modifications, item.modifications) && _.isEqual(i.fabrics, item.fabrics) && i.typeId == item.typeId;
                });

                // if an item exist with the same configuration, edit qty of cart item
                if (cartItem != undefined) 
                    cartItem.qty++;
                else                        // Else, add new item to the cart
                    app.store.cart.push(item);
                
                this.snackbarText = 'Item added to cart'                
            }

            this.snackbar = true;
        },
        validate: function () {

            let mustHaveModifications = null;
            let mustHaveFabrics = null;

            switch( app.store.typeId ) {
                case 1:
                    mustHaveModifications = ['buttons','lapel','handstitches','sleeve-buttons','pockets','pocket-type','lapel-width','vent','fit'];
                    mustHaveFabrics = ['lining','coatbody','upperlapel','lowerlapel','buttons','pockets'];
                    break;
                case 2:
                    mustHaveModifications = ['back-pocket','satin-tape'];
                    mustHaveFabrics = ['pantbody','buttons','satintape'];
                    break;
                case 3:
                    mustHaveModifications = ['neck-type','cuffs','french-cuffs','contrast','pocket','shirt-body','pleats'];
                    mustHaveFabrics = ['body','contrast','buttons'];
                    break;
                case 4:
                    mustHaveModifications = ['type','pockets'];
                    mustHaveFabrics = ['vestbody','lining','pockets','buttons'];
            }

            let missingMods = [];
            let missingFabs = [];
            // Calculate missing modifications
            _.forEach(mustHaveModifications, function(mod) {
                let found = app.store.modifications.find(function(item) {
                    return item.category == mod;
                });
                if( !found ) {

                    // Validation exceptions
                    // If the jacket has no pockets, don't make the pocket type required
                    let pocketVal = app.store.modifications.find(function(item) {
                        return item.category == 'pockets';
                    });
                    if( mod == 'pocket-type' && pocketVal != undefined && pocketVal.optId == 23 ) {
                        return;
                    }

                    missingMods.push(mod);
                }
            });
            // Calculate missing fabrics
            _.forEach(mustHaveFabrics, function(fab) {
                let found = app.store.fabrics.find(function(item) {
                    return item.fabric_option == fab;
                });
                if( !found ) {

                    // Validation exceptions
                    // If the jacket has no pockets, don't make the pocket fabric required
                    let pocketVal = app.store.modifications.find(function(item) {
                        return item.category == 'pockets';
                    });
                    if( fab == 'pockets' && pocketVal != undefined && (pocketVal.optId == 23 || pocketVal.optId == 104) ) {
                        return;
                    }

                    // When the jacket has a shawl lapel, the lower lapel fabric is not required
                    let lapelVal = app.store.modifications.find(function(item) {
                        return item.category == 'lapel';
                    });
                    if( fab == 'lowerlapel' && lapelVal != undefined && lapelVal.optId == 7 ) {
                        return;
                    }

                    // When the pants doesn't have satin tape, dont validate the satin tape fabric
                    let satinTapeVal = app.store.modifications.find(function(item) {
                        return item.category == 'satin-tape';
                    });
                    if( fab == 'satintape' && satinTapeVal != undefined && satinTapeVal.optId == 93 ) {
                        return;
                    }

                    missingFabs.push(fab);
                }
            });

            let result = {
                valid: !(missingMods.length > 0 || missingFabs.length > 0),
                missingModifications: missingMods,
                missingFabrics: missingFabs
            }

            return result;
        },
        validateCouture: function() {
            let valid = true;
            if ( this.couturePrice.length < 1 )
                valid = false;

            // this.modCategories.forEach(mod => {
            //     if( mod.description.length < 1 )
            //         valid = false;
            // });

            this.coutureValidation = !valid;
            return valid;
        },
        toTitleCase: function (str) {
            return str.replace(/\-+/g, ' ').replace(/\w\S*/g, function(txt){
                return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();
            });
        },
        priceChange: function(e, item) {
            // Allow: backspace, delete, tab, escape, enter and .
            if ($.inArray(e.keyCode, [46, 8, 9, 27, 13, 110, 190]) !== -1 ||
                // Allow: Ctrl/cmd+A
            (e.keyCode == 65 && (e.ctrlKey === true || e.metaKey === true)) ||
                // Allow: Ctrl/cmd+C
            (e.keyCode == 67 && (e.ctrlKey === true || e.metaKey === true)) ||
                // Allow: Ctrl/cmd+X
            (e.keyCode == 88 && (e.ctrlKey === true || e.metaKey === true)) ||
                // Allow: home, end, left, right
            (e.keyCode >= 35 && e.keyCode <= 39)) {
                // let it happen, don't do anything
                return;
            }

            // Ensure that it is a number and stop the keypress
            if ((e.shiftKey || (e.keyCode < 48 || e.keyCode > 57)) && (e.keyCode < 96 || e.keyCode > 105)) {
                e.preventDefault();
            }
        },
    }
});