var search = Vue.component("m2m-search-order", {
    template: `
    <v-container fluid style="height:100%;">
        <v-layout column>
            <v-flex xs12>
                <div class="text-xs-center">
                    <img src="../assets/images/LeonardoFullLogo.svg" width="200"></img>
                </div>
            </v-flex>
            <v-flex xs12>
                <v-layout align-center justify-center row fill-height v-if="order == null"> 
                    <v-flex md12 sm12 lg6>
                        <v-card>
                            <v-card-title class="headline">
                                Search for your order
                            </v-card-title>
                            <v-card-text>
                                <v-form ref="form" v-model="valid">
                                    <v-layout row wrap>
                                        <v-flex xs12 sm12 md12 lg5 xl5 class="mr-3">
                                            <p class="subheading">Here you can pay your Made To Measure order balance. Please provide the following information to look for your order details</p>
                                            <ol>
                                                <li class="subheading">Enter your order number</li>
                                                <li class="subheading">Enter the phone number or email associated with your order</li>
                                            </ol>
                                        </v-flex>
                                        <v-flex  xs12 sm12 md12 lg6 xl6>
                                            <v-layout column>
                                                <v-flex xs6>
                                                    <v-text-field v-model="orderId" label="Order Number" :rules="orderIdRules" type="tel" required></v-text-field>
                                                </v-flex>
                                                <div class="pa-2 grey lighten-3" style="border-radius: 5px;">
                                                    <v-flex xs12>
                                                        <v-text-field v-model="email" label="Email"></v-text-field>
                                                    </v-flex>
                                                    <v-flex xs12>
                                                        <v-layout align-center justify-center row fill-height>
                                                            <v-flex xs4>
                                                                <v-divider></v-divider>
                                                            </v-flex>
                                                            <v-flex xs4 class="ml-1">
                                                                <p class="text-xs-center subheading">OR</p>
                                                            </v-flex>
                                                            <v-flex xs4 class="ml-1">
                                                                <v-divider></v-divider>
                                                            </v-flex>
                                                        </v-layout>
                                                    </v-flex>
                                                    <v-flex xs12>
                                                        <v-text-field v-model="phone" label="Phone Number" mask="phone" type="tel"></v-text-field>
                                                    </v-flex>
                                                </div>
                                            </v-layout>
                                        </v-flex>
                                    </v-layout>
                                </v-form>
                            </v-card-text>

                            <v-divider></v-divider>

                            <v-card-actions>
                                <v-spacer></v-spacer>
                                <v-btn color="primary" flat @click="findOrder">
                                    Find Order
                                </v-btn>
                            </v-card-actions>
                        </v-card>
                    </v-flex>
                </v-layout>

                <!-- When order is found -->
                <v-layout align-center justify-center row fill-height v-if="order != null">
                    <v-flex md12 sm12 lg6>
                        <v-card>
                            <v-card-title class="headline">
                                <v-btn small @click="order = null"><v-icon>arrow_back</v-icon> BACK</v-btn>
                            </v-card-title>
                            <v-card-text>
                                <v-layout row wrap>
                                    <v-flex xs12 sm12 md12 lg12 xl6>
                                        <m2m-payment-processor title="Payment Information" :enableIcon="true" :total="parseFloat(order.balance)" 
                                            :isDialog="true" v-if="order.balance > 0" :orderId="order.order_id" @close="order = null" 
                                            :allowedMethods="['Stripe']" @processed="onPaymentProcessed($event)"></m2m-payment-processor>
                                    </v-flex>
                                    <v-flex xs12 sm12 md12 lg12 xl6>
                                        <div class="pa-2 grey lighten-3" style="border-radius: 5px;">
                                            <p class="title">Order Information</p>
                                            <v-layout row wrap>
                                                <v-flex xs5 class="mr-2">
                                                    Order Number:
                                                </v-flex>
                                                <v-flex xs5>
                                                    {{ this.order.order_id }}
                                                </v-flex>
                                                <v-flex xs5 class="mr-2">
                                                    Order Date:
                                                </v-flex>
                                                <v-flex xs5>
                                                    {{ this.order.order_date }}
                                                </v-flex>
                                                <v-flex xs5 class="mr-2">
                                                    Customer:
                                                </v-flex>
                                                <v-flex xs5>
                                                    {{ this.order.customer_fullname }}
                                                </v-flex>
                                                <v-flex xs5 class="mr-2">
                                                    Status:
                                                </v-flex>
                                                <v-flex xs5>
                                                    {{ this.order.status }}
                                                </v-flex>
                                                <v-flex xs5 class="mr-2">
                                                    Order Total:
                                                </v-flex>
                                                <v-flex xs5>
                                                    {{ this.formatMoney( this.order.total ) }}
                                                </v-flex>
                                                <v-flex xs5 class="mr-2">
                                                    Balance:
                                                </v-flex>
                                                <v-flex xs5>
                                                    <span class="green--text darken-2 body-2">{{ this.formatMoney( this.order.balance ) }}</span> <br>
                                                    <a class="caption" @click="togglePaymentHistory">{{ togglePaymentHistoryLabel }}</a>
                                                </v-flex>
                                                <v-flex xs12 class="mt-3" v-if="prevPayments">
                                                    <p class="title">Payments History</p>
                                                    <table class="v-datatable v-table theme--light">
                                                        <thead>
                                                            <tr>
                                                                <th class="column text-xs-left">Date</th>
                                                                <th class="column text-xs-left">Method</th>
                                                                <th class="column text-xs-right">Amount</th>
                                                            </tr>
                                                        </thead>
                                                        <tbody>
                                                            <tr v-for="(payment, index) in order.payments" :key="index">
                                                                <td>{{ payment.payment_date }}</td>
                                                                <td>{{ payment.payment_method }}</td>
                                                                <td class="text-xs-right">{{ formatMoney( payment.payment_amount ) }}</td>
                                                            </tr>
                                                        </tbody>
                                                        <tfoot>
                                                            <tr>
                                                                <td colspan="2" class="body-2 grey lighten-4">Total</td>
                                                                <td class="text-xs-right body-2 grey lighten-4">{{ formatMoney( this.totalPaymentAmount ) }}</td>
                                                            </tr>
                                                        </tfoot>
                                                    </table>
                                                </v-flex>
                                            </v-layout>
                                        </div>
                                    </v-flex>
                                </v-layout>
                            </v-card-text>
                        </v-card>
                    </v-flex>
                </v-layout>
            </v-flex>
        </v-layout>
        <v-dialog v-model="dialog" scrollable  max-width="500">
            <v-card>
                <v-card-title class="headline red darken-1"><v-icon>error</v-icon> Missing Information</v-card-title>
                <v-divider></v-divider>
                <v-card-text>You're missing one or more fields.You need to provide your order number and the email or phone number associated with your order.</v-card-text>
                <v-divider></v-divider>
                <v-card-actions>
                    <v-btn color="grey darken-1" flat @click.native="dialog = false" style="margin-left:auto; margin-right:0;">Ok</v-btn>
                </v-card-actions>
            </v-card>
        </v-dialog>

        <!-- Success Dialog -->
        <v-dialog v-model="successDialog" scrollable  max-width="500">
            <v-card>
                <v-card-title class="headline green darken-1"><v-icon color="white">check_circle_outline</v-icon> Thank you!</v-card-title>
                <v-divider></v-divider>
                <v-card-text>Your payment has been received. Your current balance is <strong v-if="order != null">{{ this.formatMoney( this.order.balance - this.amountPaid ) }}</strong>.</v-card-text>
                <v-divider></v-divider>
                <v-card-actions>
                    <v-btn color="grey darken-1" flat @click.native="closeSuccessDialog" style="margin-left:auto; margin-right:0;">Ok</v-btn>
                </v-card-actions>
            </v-card>
        </v-dialog>
    </v-container>`,
    data: function() {
        return { 
            orderId: '',
            email: '',
            phone: '',
            orderIdRules: [
                v => !!v || 'Required',
                v => /[0-9]{4,}/g.test(v) || 'Order number must be valid'
            ],
            formHasError: false,
            dialog: false,
            order: null,
            valid: false,
            prevPayments: false,
            successDialog: false,
            amountPaid: 0,
        }
    },
    created () {
        
    },
    mounted: function() {

    },
    computed: {
        togglePaymentHistoryLabel: function() {
            if ( this.prevPayments )
                return 'Hide payments history';
            
            return 'View payments history';
        },
        totalPaymentAmount: function() {
            let total = 0;
            this.order.payments.forEach(p => {
                total += parseFloat(p.payment_amount);
            });

            return total;
        }
    },
    methods: {
        findOrder: function() {
            this.validateForm();
            if ( this.formHasError ) {
                this.dialog = true;
                return;
            }

            let attribute = this.email;
            if( this.email.length < 1 )
                attribute = this.phone;

            // Get order information
            axios
                .get( app.dataUrl + '?action=odrpmtsmy/' + this.orderId + '/' + attribute, { headers: { Authorization: app.authKey } })
                .then( response => { 
                    if(response.data.status == false) {
                        alert(response.data.message);
                        return;
                    }

                    this.order = response.data.data;
                });
        },
        validateForm: function() {
            let form = this.$refs.form.validate();

            if( form && ( this.email.length > 0 || this.phone.length > 0 ) )
                this.formHasError = false;
            else
                this.formHasError = true;
        },
        formatMoney: function(n, c, d, t) {
            var c = isNaN(c = Math.abs(c)) ? 2 : c,
              d = d == undefined ? "." : d,
              t = t == undefined ? "," : t,
              s = n < 0 ? "-" : "",
              i = String(parseInt(n = Math.abs(Number(n) || 0).toFixed(c))),
              j = (j = i.length) > 3 ? j % 3 : 0;
          
            return '$' + s + (j ? i.substr(0, j) + t : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.abs(n - i).toFixed(c).slice(2) : "");
        },
        togglePaymentHistory: function() {
            if ( this.prevPayments )
                this.prevPayments = false;
            else
                this.prevPayments = true;
        },
        onPaymentProcessed: function(payment) {
            this.successDialog = true;
            this.amountPaid = payment.amount;
        },
        closeSuccessDialog: function() {
            this.successDialog = false;
            this.amountPaid = 0;
            this.order = null;
        }
    }
});